import PySimpleGUI as sg


class FileBrowser:
    def get_folder_path(self):
        path = self.execute_folder()
        self.close()
        return path

    def get_file_path(self):
        path = self.execute()
        self.close()
        return path

    def execute(self):
        layout = [
            [sg.T("")],
            [sg.Text("Choose a file: "), sg.Input(), sg.FileBrowse(key="-IN-")],
            [sg.Button("Upload")],
        ]

        ###Building Window
        self.window = sg.Window("File Browser", layout, keep_on_top=True)

        while True:
            event, values = self.window.read()
            if event == sg.WIN_CLOSED or event == "Exit":
                break
            elif event == "Upload":
                value = values["-IN-"]
                break
        try:
            return value
        except:
            return None

    def execute_folder(self):
        layout = [
            [sg.Text("Select a folder:")],
            [sg.Input(size=(40, 1)), sg.FolderBrowse(key="-FOLDER-")],
            [sg.Button("Select"), sg.Button("Exit")],
        ]

        self.window = sg.Window("Folder Browser", layout)

        while True:
            event, values = self.window.read()
            if event == sg.WIN_CLOSED or event == "Exit":
                break
            elif event == "Select":
                # Get the selected folder path from the Input element
                selected_folder_path = values["-FOLDER-"]
                # Do something with the selected folder path (e.g., print it)
                return selected_folder_path

    def close(self):
        try:
            self.window.close()
        except:
            pass


if __name__ == "__main__":
    print(FileBrowser().get_folder_path())



# import parsel

import base64
import json
import os
from urllib.parse import urlparse

import requests
from libs.in_win import config_path, inside_windows
from libs.V2RayURL2Config import generate_v2rayconfig_with_name


class Subscriptions:
    def __init__(self, URL="") -> None:
        self.URL = URL

    def create_name_from_url(self, url):
        # Parse the URL to extract its components
        parsed_url = urlparse(url)

        # Extract the website name (domain) from the URL
        website_name = parsed_url.netloc.split(".")[0]

        # Extract other parts of the URL (path and query) if available
        path_segments = parsed_url.path.split("/")
        other_parts = [segment for segment in path_segments if segment]

        # Combine the website name and other parts to form the name
        name_parts = [website_name] + other_parts
        name = "_".join(name_parts)

        return name

    def put_v2ray_subscription_in_folder(
        self, subscription_name: str, config_name: str, json_config: json
    ):
        path = f"{config_path()}\\v2ray_profiles\\subscriptions\\{subscription_name}"

        file_path = f"{path}\\{config_name}.json"
        if not inside_windows():
            file_path = file_path.replace("\\", "/")
        with open(file_path, "w") as file:
            json.dump(json_config, file, indent=2)

    def put_gost_subscription_in_folder(
        self, subscription_name: str, config_name: str, json_config: json
    ):
        path = f"{config_path()}\\gost_profiles\\subscriptions\\{subscription_name}"

        try:
            os.makedirs(path)
        except Exception as e:
            print("Err occured 744: ", str(e))

        file_path = f"{path}\\{config_name}.json"
        if not inside_windows():
            file_path = file_path.replace("\\", "/")
        with open(file_path, "w") as file:
            json.dump(json_config, file, indent=2)

    def make_subscription(self):
        try:
            text_output = requests.get(self.URL).text.strip()
        except:
            return False
        responses = text_output.split("\n")

        # responses = [
        #     r"trojan://LjWyuE2d8M@hels.ddns.net:44838?security=reality&sni=yahoo.com&fp=firefox&pbk=8LIuGGsdhR59qjyRALAmGKNuKVlyH3t8OqJmRRdyKl4&sid=92cd56b1&spx=%2F&type=grpc"
        # ]
        subscription_name = (
            self.create_name_from_url(self.URL).replace(" ", "_").replace("|", "_")
        )  # "isharifi"
        self.make_folder_ready(subscription_name)

        for response in responses:
            try:
                if "gost://" not in response:
                    json_out, name = generate_v2rayconfig_with_name(response)
                    name = name.replace(" ", "_").replace("|", "_")
                    self.put_v2ray_subscription_in_folder(
                        subscription_name, name, json_out
                    )
                else:
                    content64 = response.replace("gost://", "")
                    content = str(base64.b64decode(content64).decode("utf-8"))
                    json_out = json.loads(content)
                    name = f'{json_out["remote_protocol"]}_{json_out["remote_port"]}'
                    self.put_gost_subscription_in_folder(
                        subscription_name, name, json_out
                    )
            except Exception as e:
                print(e)
        return True

    def make_folder_ready(self, subscription_name):
        ################################temporary remove all subscriptions
        self.delete_subscription_folder()
        ################################
        path = f"{config_path()}\\v2ray_profiles\\subscriptions\\{subscription_name}"
        cmd = f"rmdir /s /q {path}"
        if not inside_windows():
            path = path.replace("\\", "/")
            cmd = f"rm -rf {path}"
        os.popen(cmd).read()
        if not inside_windows():
            path = path.replace("\\", "/")
        cmd = f"mkdir {path}"
        if not inside_windows():
            cmd = f"mkdir -p {path}"
        os.popen(cmd).read()

    def delete_subscription_folder(self):
        path = f"{config_path()}\\v2ray_profiles\\subscriptions\\"
        cmd = f"rmdir /s /q {path}"
        if not inside_windows():
            path = path.replace("\\", "/")
            cmd = f"rm -rf {path}"
        os.popen(cmd).read()

        path = f"{config_path()}\\gost_profiles\\subscriptions\\"
        cmd = f"rmdir /s /q {path}"
        if not inside_windows():
            path = path.replace("\\", "/")
            cmd = f"rm -rf {path}"
        os.popen(cmd).read()


if __name__ == "__main__":
    URL = "https://isharifi.ir/codes/v2ray"
    Subscriptions(URL).make_subscription()



import json
import os

from libs.in_win import config_path, inside_windows


class RefereshEditPage:
    def __init__(self, filename, group="") -> None:
        self.page_data = dict()
        self.remark = filename
        self.group = group
        data = self.read_content_of_config_file_as_json()
        self.extract_configs(data)

    def get_editpage_content(self) -> dict:
        self.page_data["remark"] = self.remark
        return self.page_data

    def extract_configs(self, data: json):
        self.page_data["protocol"] = data["outbounds"][0]["protocol"]
        self.page_data["network"] = data["outbounds"][0]["streamSettings"]["network"]
        self.page_data["user_security"] = ""
        if "trojan" in self.page_data["protocol"]:
            self.page_data["server_address"] = data["outbounds"][0]["settings"][
                "servers"
            ][0]["address"]
            self.page_data["method"] = data["outbounds"][0]["settings"]["servers"][0][
                "method"
            ]
            self.page_data["ota"] = data["outbounds"][0]["settings"]["servers"][0][
                "ota"
            ]
            self.page_data["password"] = data["outbounds"][0]["settings"]["servers"][0][
                "password"
            ]
            self.page_data["port"] = data["outbounds"][0]["settings"]["servers"][0][
                "port"
            ]
        elif ("vless" in self.page_data["protocol"]) or (
            "vmess" in self.page_data["protocol"]
        ):
            self.page_data["server_address"] = data["outbounds"][0]["settings"][
                "vnext"
            ][0]["address"]
            self.page_data["port"] = data["outbounds"][0]["settings"]["vnext"][0][
                "port"
            ]
            self.page_data["security"] = data["outbounds"][0]["settings"]["vnext"][0][
                "users"
            ][0]["security"]

            self.page_data["uuid"] = data["outbounds"][0]["settings"]["vnext"][0][
                "users"
            ][0]["id"]

        if self.page_data["network"] == "grpc":
            try:
                self.page_data["grpc_serviceName"] = data["outbounds"][0][
                    "streamSettings"
                ]["grpcSettings"]["serviceName"]
            except:
                pass

        try:
            self.page_data["security"] = data["outbounds"][0]["streamSettings"][
                "security"
            ]
            if self.page_data["security"] == "reality":
                self.page_data["serverName"] = data["outbounds"][0]["streamSettings"][
                    "realitySettings"
                ]["serverName"]
                self.page_data["fingerprint"] = data["outbounds"][0]["streamSettings"][
                    "realitySettings"
                ]["fingerprint"]
                self.page_data["publicKey"] = data["outbounds"][0]["streamSettings"][
                    "realitySettings"
                ]["publicKey"]
                self.page_data["shortId"] = data["outbounds"][0]["streamSettings"][
                    "realitySettings"
                ]["shortId"]
                self.page_data["spiderX"] = data["outbounds"][0]["streamSettings"][
                    "realitySettings"
                ]["spiderX"]
            # elif self.security == "reality":
            elif self.page_data["security"] == "tls":
                self.page_data["serverName"] = data["outbounds"][0]["streamSettings"][
                    "tlsSettings"
                ]["serverName"]
                self.page_data["fingerprint"] = data["outbounds"][0]["streamSettings"][
                    "tlsSettings"
                ]["fingerprint"]
                arr_alpn = data["outbounds"][0]["streamSettings"]["tlsSettings"]["alpn"]
                text = ""
                for alpn in arr_alpn:
                    text = text + "," + alpn
                self.page_data["alpn"] = text

        except:
            self.page_data["security"] = "none"

    def read_content_of_config_file_as_json(self) -> json:
        group = self.group
        if len(group) > 1:
            group_path = f"\\subscriptions\\{group}"
        else:
            group_path = ""
        ################

        json_config_path = f"{config_path()}\\v2ray_profiles{group_path}\\{self.remark}"
        if not inside_windows():
            json_config_path = json_config_path.replace("\\", "/")

        with open(json_config_path, "r") as json_file:
            # Load the JSON data
            data = json.load(json_file)
        return data



import os

# Generate the QR code
import pyperclip
import PySimpleGUI as sg
import qrcode,platform
from libs.in_win import config_path, inside_windows


class QRCode:
    def __init__(self, data) -> None:
        qr = qrcode.QRCode(
            version=1,
            error_correction=qrcode.constants.ERROR_CORRECT_H,
            box_size=10,
            border=4,
        )
        qr.add_data(data)
        qr.make(fit=True)
        qr_image = qr.make_image(fill_color="black", back_color="white")
        qr_image = qr_image.resize((400, 400))

        # Save the QR code as an image file
        if inside_windows():
            image_file = f"{os.getenv('TEMP')}\\qrcode.png"  # Provide the desired filename and extension
        else:
            if platform.system()=="Darwin":
                temp_dir=f'{os.popen("echo $TMPDIR").read().strip()}'
            elif platform.system()=="Linux":
                temp_dir=f"{os.popen('MYTMPDIR=$(mktemp -d);echo $MYTMPDIR').read().strip()}"
            image_file = f"{temp_dir}/qrcode.png"  # Provide the desired filename and extension

        qr_image.save(image_file)

        # Create the PySimpleGUI window
        layout = [
            [sg.Image(image_file, key="-IMAGE-", size=(400, 400))],
            [sg.Multiline(data, size=(400, 5))],
            [sg.Button("Copy to Clipboard", key="copy"), sg.Button("Close")],
        ]

        window = sg.Window("QR Code", layout, size=(450, 550), keep_on_top=True)

        # Event loop to handle window events
        while True:
            event, _ = window.read()
            if event == sg.WINDOW_CLOSED or event == "Close":
                break
            elif "copy" in event:
                pyperclip.copy(data)

        window.close()


if __name__ == "__main__":
    data = "trojan://jJxdKznjbv@hels.ddns.net:2096?security=reality&sni=yahoo.com&fp=firefox&pbk=8LIuGGsdhR59qjyRALAmGKNuKVlyH3t8OqJmRRdyKl4&sid=92cd56b1&spx=%2F&type=grpc#reality-trojan-2096"  # Replace with your desired data
    QRCode(data)



import json
import os
import platform
import signal
import subprocess
import threading
import time

import psutil
from libs.in_win import config_path, inside_windows


class ConnectGost:
    def __init__(self, config_path, localport=8080) -> None:
        self.config_path = config_path
        self.localport = localport
        self.sni = False
        self.__profile_from_config_file()
        self.__profiler_reader()
        self.enable_loops = True
        self.gost_process = None

    def __profile_from_config_file(self):
        with open(self.config_path, "r") as json_file:
            # Load the JSON data from the file
            self.profile = json.load(json_file)

    def __profiler_reader(self):
        self.remote_protocol = self.profile["remote_protocol"]
        self.remote_user = self.profile["remote_user"]
        self.remote_password = self.profile["remote_password"]
        self.remote_address = self.profile["remote_address"]
        self.remote_port = self.profile["remote_port"]
        try:
            self.sni = self.profile["sni"]
        except:
            pass

    def __cmd_generator(self):
        bin_address = self.bin_address_generator()

        if not self.sni:
            cmd = f"{bin_address} -D -L=auto://0.0.0.0:{self.localport} -F={self.remote_protocol}://{self.remote_user}:{self.remote_password}@{self.remote_address}:{self.remote_port}"
        else:
            cmd = f'{bin_address} -D -L="auto://0.0.0.0:{self.localport}" -F="{self.remote_protocol}://{self.remote_user}:{self.remote_password}@:{self.remote_port}?host={self.sni}"'
        # else:
        #     if not self.sni:
        #         cmd = f"bin/mac/gost/igost -D -L=auto://0.0.0.0:{self.localport} -F={self.remote_protocol}://{self.remote_user}:{self.remote_password}@{self.remote_address}:{self.remote_port}"
        #     else:
        #         cmd = f'bin/mac/gost/igost -D -L="auto://0.0.0.0:{self.localport}" -F="{self.remote_protocol}://{self.remote_user}:{self.remote_password}@:{self.remote_port}?host={self.sni}"'

        return cmd

    def bin_address_generator(self):
        # if inside_windows():
        #     pf = "win"
        # elif platform.system() == "Linux":
        #     pf = "linux"

        # elif platform.system() == "Darwin":
        #     pf = "mac"

        # bin_address = f"bin\\{pf}\\gost\\gost"
        bin_address = f"{config_path()}\\bin\\gost"
        if not inside_windows():
            bin_address = bin_address.replace("\\", "/")
        return bin_address

    def kill(self):
        self.enable_loops = False
        if self.gost_process:
            # self.gost_process.kill()
            self.kills_pid(self.gost_process.pid)
            self.gost_process.terminate()
            # self.gost_process.send_signal(signal.CTRL_C_EVENT)
            kill_cmd = f"taskkill /f /pid {self.gost_process.pid}"
            print(kill_cmd)
            os.popen(kill_cmd)

        # os.popen("taskkill /f /im igost.exe").read()
        self.gost_thread.join(1)

    def connect(self):
        cmd = self.__cmd_generator()
        self.gost_thread = threading.Thread(
            target=self.__thread_run_read_gost, args=(cmd,), daemon=True
        )
        self.gost_thread.start()
        return self.gost_thread

    def __thread_run_read_gost(self, cmd):
        self.enable_loops = True
        print("thread_run_read_gost is ran")

        print("cmd before: " + cmd)

        self.gost_process = subprocess.Popen(
            cmd,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            shell=True,
        )

        # tasklist = os.popen("tasklist").read()

        while self.enable_loops:
            line = self.gost_process.stderr.readline().strip().decode("utf-8")
            if len(line) < 3:
                time.sleep(0.5)
                continue
            print(line)

    def kills_pid(self, pid):
        parent = psutil.Process(pid)
        for child in parent.children(recursive=True):
            child.kill()
        parent.kill()


if __name__ == "__main__":
    gi = ConnectGost("gost1.json")
    gi.connect()
    while True:
        time.sleep(5)



import json
import os
import platform
import subprocess
import threading
import time

import psutil
from libs.in_win import config_path, inside_windows


class ConnectV2Ray:
    def __init__(self, config_file_path, localport=7595) -> None:
        print("v2ray started!")
        self.enable_loops = False
        self.localport = localport

        self.config_file_path = config_file_path
        self.__referesh_localport_remoteport()

        self.v2ray_thread = None
        self.v2ray_process = None

    def __referesh_localport_remoteport(self):
        with open(f"{self.config_file_path}", "r") as json_file:
            # Load the JSON data from the file
            data = json.load(json_file)
        data["log"]["loglevel"] = "debug"
        data["inbounds"][0]["port"] = int(self.localport)
        try:
            data["inbounds"][1]["port"] = int(self.localport) + 1
        except:
            pass

        try:
            a = data["outbounds"][0]["settings"]["vnext"][0]["port"]
            data["outbounds"][0]["settings"]["vnext"][0]["port"] = int(a)
        except:
            a = data["outbounds"][0]["settings"]["servers"][0]["port"]
            data["outbounds"][0]["settings"]["servers"][0]["port"] = int(a)

        with open(f"{self.config_file_path}", "w") as json_file:
            # Write the JSON data to the file
            json.dump(data, json_file)

    def connect(self):
        cmd = self.cmd_generator()
        print(cmd)
        self.v2ray_thread = threading.Thread(
            target=self.thread_run_read_v2ray, args=(cmd,), daemon=True
        )
        self.v2ray_thread.start()
        return self.v2ray_thread

    def kill_pid(self, pid):
        parent = psutil.Process(pid)
        for child in parent.children(recursive=True):
            child.kill()
        parent.kill()

    def kill(self):
        self.enable_loops = False
        if self.v2ray_process:
            self.kill_pid(self.v2ray_process.pid)
            self.v2ray_process.terminate()
            # self.v2ray_process.send_signal(signal.CTRL_C_EVENT)
            os.popen(f"taskkill /f /pid {self.v2ray_process.pid}")
            self.v2ray_process.kill()
        # os.popen("taskkill /f /im xray.exe").read()
        self.v2ray_thread.join(1)

    def cmd_generator(
        self,
    ) -> str:
        # cmd = f"bin\\v2ray\\xray.exe run -c %appdata\\roaming%\v2rayp\configs\\{self.config_filename}"
        if inside_windows():
            cmd = f"{config_path()}\\bin\\xray.exe --config={self.config_file_path}"
        else:
            cmd = f"chmod +x {config_path()}/bin/xray && {config_path()}/bin/xray --config={self.config_file_path}"

            # if platform.system() == "Darwin":
            #     cmd = f"chmod +x  && bin/mac/v2ray/xray --config={self.config_file_path}"
            # elif platform.system() == "Linux":
            #     cmd = f"chmod +x bin/mac/v2ray/xray && bin/linux/v2ray/xray --config={self.config_file_path}"

        return cmd

    def thread_run_read_v2ray(self, cmd):
        self.enable_loops = True
        print("thread_run_read_v2ray is ran")

        err_cnt = 0

        print("cmd before: " + cmd)

        # tasklist = ""
        # while "xray" not in tasklist:
        self.v2ray_process = subprocess.Popen(
            cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE
        )
        # tasklist = os.popen("tasklist").read()

        print("next line")
        while self.enable_loops:
            line = self.v2ray_process.stdout.readline().strip().decode("utf-8")
            if len(line) < 3:
                time.sleep(0.5)
                continue
            print(line)
            if "failed to find an available destination" in line:
                err_cnt += 1

            elif "via" in line:
                err_cnt = 0



import argparse
import base64
import json
import os
import re
import sys
from urllib.parse import parse_qs, unquote, urlparse

sys.path.append("v2rayp")

from libs.GenerateRandomName import generate_random_alphanumeric

DEFAULT_PORT = 443
DEFAULT_SECURITY = "auto"
DEFAULT_LEVEL = 8
DEFAULT_NETWORK = "tcp"

TLS = "tls"
REALITY = "reality"
HTTP = "http"


class EConfigType:
    class VMESS:
        protocolScheme = "vmess://"
        protocolName = "vmess"

    class CUSTOM:
        protocolScheme = ""
        protocolName = ""

    class SHADOWSOCKS:
        protocolScheme = "ss://"
        protocolName = "ss"

    class SOCKS:
        protocolScheme = "socks://"
        protocolName = "socks"

    class VLESS:
        protocolScheme = "vless://"
        protocolName = "vless"

    class TROJAN:
        protocolScheme = "trojan://"
        protocolName = "trojan"

    class WIREGUARD:
        protocolScheme = "wireguard://"
        protocolName = "wireguard"

    class FREEDOM:
        protocolScheme = "freedom://"
        protocolName = "freedom"

    class BLACKHOLE:
        protocolScheme = "blackhole://"
        protocolName = "blackhole"


class DomainStrategy:
    AsIs = "AsIs"
    UseIp = "UseIp"
    IpIfNonMatch = "IpIfNonMatch"
    IpOnDemand = "IpOnDemand"


class Fingerprint:
    randomized = "randomized"
    randomizedalpn = "randomizedalpn"
    randomizednoalpn = "randomizednoalpn"
    firefox_auto = "firefox_auto"
    chrome_auto = "chrome_auto"
    ios_auto = "ios_auto"
    android_11_okhttp = "android_11_okhttp"
    edge_auto = "edge_auto"
    safari_auto = "safari_auto"
    _360_auto = "360_auto"
    qq_auto = "qq_auto"


class LogBean:
    access: str
    error: str
    loglevel: str
    dnsLog: bool

    def __init__(self, access: str, error: str, loglevel: str, dnsLog: bool) -> None:
        self.access = access
        self.error = error
        self.loglevel = loglevel
        self.dnsLog = dnsLog


class InboundBean:
    class SniffingBean:
        enabled: bool
        destOverride: list[str]  # str
        metadataOnly: bool

        def __init__(
            self, enabled: bool, destOverride: list[str], metadataOnly: bool
        ) -> None:
            self.enabled = enabled
            self.destOverride = destOverride
            self.metadataOnly = metadataOnly

    class InSettingsBean:
        auth: str = None
        udp: bool = None
        userLevel: int = None
        address: str = None
        port: int = None
        network: str = None

        def __init__(
            self,
            auth: str = None,
            udp: bool = None,
            userLevel: int = None,
            address: str = None,
            port: int = None,
            network: str = None,
        ) -> None:
            self.auth = auth
            self.udp = udp
            self.userLevel = userLevel
            self.address = address
            self.port = port
            self.network = network

    tag: str
    port: int
    protocol: str
    listen: str
    settings: any
    sniffing: SniffingBean
    streamSettings: any
    allocate: any

    def __init__(
        self,
        tag: str,
        port: int,
        protocol: str,
        listen: str,
        settings: any,
        sniffing: SniffingBean,
        streamSettings: any,
        allocate: any,
    ) -> None:
        self.tag = tag
        self.port = port
        self.protocol = protocol
        self.listen = listen
        self.settings = settings
        self.sniffing = sniffing
        self.streamSettings = streamSettings
        self.allocate = allocate


class OutboundBean:
    class OutSettingsBean:
        class VnextBean:
            class UsersBean:
                id: str = ""
                alterId: int = None
                security: str = DEFAULT_SECURITY
                level: int = DEFAULT_LEVEL
                encryption: str = ""
                flow: str = ""

                def __init__(
                    self,
                    id: str = "",
                    alterId: int = None,
                    security: str = DEFAULT_SECURITY,
                    level: int = DEFAULT_LEVEL,
                    encryption: str = "",
                    flow: str = "",
                ) -> None:
                    self.id = id
                    self.alterId = alterId
                    self.security = security
                    self.level = level
                    self.encryption = encryption
                    self.flow = flow

            address: str = ""
            port: int = DEFAULT_PORT
            users: list[UsersBean]  # UsersBean

            def __init__(
                self,
                address: str = "",
                port: int = DEFAULT_PORT,
                users: list[UsersBean] = [],
            ) -> None:
                self.address = address
                self.port = port
                self.users = users

        class ServersBean:
            class SocksUsersBean:
                user: str = ""
                # @SerializedName("pass")
                _pass: str = ""
                level: int = DEFAULT_LEVEL

                def __init__(
                    self, user: str = "", _pass: str = "", level: int = DEFAULT_LEVEL
                ) -> None:
                    self.user = user
                    self._pass = _pass
                    self.level = level

            address: str = ""
            method: str = "chacha20-poly1305"
            ota: bool = False
            password: str = ""
            port: int = DEFAULT_PORT
            level: int = DEFAULT_LEVEL
            email: str = None
            flow: str = None
            ivCheck: bool = None
            users: list[SocksUsersBean] = None  # SocksUsersBean

            def __init__(
                self,
                address: str = "",
                method: str = "chacha20-poly1305",
                ota: bool = False,
                password: str = "",
                port: int = DEFAULT_PORT,
                level: int = DEFAULT_LEVEL,
                email: str = None,
                flow: str = None,
                ivCheck: bool = None,
                users: list[SocksUsersBean] = None,
            ) -> None:
                self.address = address
                self.method = method
                self.ota = ota
                self.password = password
                self.port = port
                self.level = level
                self.email = email
                self.flow = flow
                self.ivCheck = ivCheck
                self.users = users

        class Response:
            type: str

            def __init__(self, type: str) -> None:
                self.type = type

        class WireGuardBean:
            publicKey: str = ""
            endpoint: str = ""

            def __init__(self, publicKey: str = "", endpoint: str = "") -> None:
                self.publicKey = publicKey
                self.endpoint = endpoint

        vnext: list[VnextBean] = None  # VnextBean
        servers: list[ServersBean] = None  # ServersBean
        response: Response = None
        network: str = None
        address: str = None
        port: int = None
        domainStrategy: str = None
        redirect: str = None
        userLevel: int = None
        inboundTag: str = None
        secretKey: str = None
        peers: list[WireGuardBean] = None  # WireGuardBean

        def __init__(
            self,
            vnext: list[VnextBean] = None,
            servers: list[ServersBean] = None,
            response: Response = None,
            network: str = None,
            address: str = None,
            port: int = None,
            domainStrategy: str = None,
            redirect: str = None,
            userLevel: int = None,
            inboundTag: str = None,
            secretKey: str = None,
            peers: list[WireGuardBean] = None,
        ) -> None:
            self.vnext = vnext
            self.servers = servers
            self.response = response
            self.network = network
            self.address = address
            self.port = port
            self.domainStrategy = domainStrategy
            self.redirect = redirect
            self.userLevel = userLevel
            self.inboundTag = inboundTag
            self.secretKey = secretKey
            self.peers = peers

    class StreamSettingsBean:
        class TcpSettingsBean:
            class HeaderBean:
                class RequestBean:
                    class HeadersBean:
                        Host: list[str] = []  # str
                        # @SerializedName("User-Agent")
                        userAgent: list[str] = None  # str
                        # @SerializedName("Accept-Encoding")
                        acceptEncoding: list[str] = None  # str
                        Connection: list[str] = None  # str
                        Pragma: str = None

                        def __init__(
                            self,
                            Host: list[str] = [],
                            userAgent: list[str] = None,
                            acceptEncoding: list[str] = None,
                            Connection: list[str] = None,
                            Pragma: str = None,
                        ) -> None:
                            self.Host = Host
                            self.userAgent = userAgent
                            self.acceptEncoding = acceptEncoding
                            self.Connection = Connection
                            self.Pragma = Pragma

                    path: list[str] = []  # str
                    headers: HeadersBean = HeadersBean()
                    version: str = None
                    method: str = None

                    def __init__(
                        self,
                        path: list[str] = [],
                        headers: HeadersBean = HeadersBean(),
                        version: str = None,
                        method: str = None,
                    ) -> None:
                        self.path = path
                        self.headers = headers
                        self.version = version
                        self.method = method

                type: str = "none"
                request: RequestBean = None

                def __init__(
                    self, type: str = "none", request: RequestBean = None
                ) -> None:
                    self.type = type
                    self.request = request

            header: HeaderBean = HeaderBean()
            acceptProxyProtocol: bool = None

            def __init__(
                self,
                header: HeaderBean = HeaderBean(),
                acceptProxyProtocol: bool = None,
            ) -> None:
                self.header = header
                self.acceptProxyProtocol = acceptProxyProtocol

        class KcpSettingsBean:
            class HeaderBean:
                type: str = "none"

                def __init__(self, type: str = "none") -> None:
                    self.type = type

            mtu: int = 1350
            tti: int = 50
            uplinkCapacity: int = 12
            downlinkCapacity: int = 100
            congestion: bool = False
            readBufferSize: int = 1
            writeBufferSize: int = 1
            header: HeaderBean = HeaderBean()
            seed: str = None

            def __init__(
                self,
                mtu: int = 1350,
                tti: int = 50,
                uplinkCapacity: int = 12,
                downlinkCapacity: int = 100,
                congestion: bool = False,
                readBufferSize: int = 1,
                writeBufferSize: int = 1,
                header: HeaderBean = HeaderBean(),
                seed: str = None,
            ) -> None:
                self.mtu = mtu
                self.tti = tti
                self.uplinkCapacity = uplinkCapacity
                self.downlinkCapacity = downlinkCapacity
                self.congestion = congestion
                self.readBufferSize = readBufferSize
                self.writeBufferSize = writeBufferSize
                self.header = header
                self.seed = seed

        class WsSettingsBean:
            class HeadersBean:
                Host: str = ""

                def __init__(self, Host: str = "") -> None:
                    self.Host = Host

            path: str = ""
            headers: HeadersBean = HeadersBean()
            maxEarlyData: int = None
            useBrowserForwarding: bool = None
            acceptProxyProtocol: bool = None

            def __init__(
                self,
                path: str = "",
                headers: HeadersBean = HeadersBean(),
                maxEarlyData: int = None,
                useBrowserForwarding: bool = None,
                acceptProxyProtocol: bool = None,
            ) -> None:
                self.path = path
                self.headers = headers
                self.maxEarlyData = maxEarlyData
                self.useBrowserForwarding = useBrowserForwarding
                self.acceptProxyProtocol = acceptProxyProtocol

        class HttpSettingsBean:
            host: list[str] = []  # str
            path: str = ""

            def __init__(self, host: list[str] = [], path: str = "") -> None:
                self.host = host
                self.path = path

        class TlsSettingsBean:
            allowInsecure: bool = False
            serverName: str = ""
            alpn: list[str] = None  # str
            minVersion: str = None
            maxVersion: str = None
            preferServerCipherSuites: bool = None
            cipherSuites: str = None
            fingerprint: str = None
            certificates: list[any] = None  # any
            disableSystemRoot: bool = None
            enableSessionResumption: bool = None
            show: bool = False
            publicKey: str = None
            shortId: str = None
            spiderX: str = None

            def __init__(
                self,
                allowInsecure: bool = False,
                serverName: str = "",
                alpn: list[str] = None,
                minVersion: str = None,
                maxVersion: str = None,
                preferServerCipherSuites: bool = None,
                cipherSuites: str = None,
                fingerprint: str = None,
                certificates: list[any] = None,
                disableSystemRoot: bool = None,
                enableSessionResumption: bool = None,
                show: bool = False,
                publicKey: str = None,
                shortId: str = None,
                spiderX: str = None,
            ) -> None:
                self.allowInsecure = allowInsecure
                self.serverName = serverName
                self.alpn = alpn
                self.minVersion = minVersion
                self.maxVersion = maxVersion
                self.preferServerCipherSuites = preferServerCipherSuites
                self.cipherSuites = cipherSuites
                self.fingerprint = fingerprint
                self.certificates = certificates
                self.disableSystemRoot = disableSystemRoot
                self.enableSessionResumption = enableSessionResumption
                self.show = show
                self.publicKey = publicKey
                self.shortId = shortId
                self.spiderX = spiderX

        class QuicSettingBean:
            class HeaderBean:
                type: str = "none"

                def __init__(self, type: str = "none") -> None:
                    self.type = type

            security: str = "none"
            key: str = ""
            header: HeaderBean = HeaderBean()

            def __init__(
                self,
                security: str = "none",
                key: str = "",
                header: HeaderBean = HeaderBean(),
            ) -> None:
                self.security = security
                self.key = key
                self.header = header

        class GrpcSettingsBean:
            serviceName: str = ""
            multiMode: bool = None

            def __init__(self, serviceName: str = "", multiMode: bool = None) -> None:
                self.serviceName = serviceName
                self.multiMode = multiMode

        network: str = DEFAULT_NETWORK
        security: str = ""
        tcpSettings: TcpSettingsBean = None
        kcpSettings: KcpSettingsBean = None
        wsSettings: WsSettingsBean = None
        httpSettings: HttpSettingsBean = None
        tlsSettings: TlsSettingsBean = None
        quicSettings: QuicSettingBean = None
        realitySettings: TlsSettingsBean = None
        grpcSettings: GrpcSettingsBean = None
        dsSettings: any = None
        sockopt: any = None

        def __init__(
            self,
            network: str = DEFAULT_NETWORK,
            security: str = "",
            tcpSettings: TcpSettingsBean = None,
            kcpSettings: KcpSettingsBean = None,
            wsSettings: WsSettingsBean = None,
            httpSettings: HttpSettingsBean = None,
            tlsSettings: TlsSettingsBean = None,
            quicSettings: QuicSettingBean = None,
            realitySettings: TlsSettingsBean = None,
            grpcSettings: GrpcSettingsBean = None,
            dsSettings: any = None,
            sockopt: any = None,
        ) -> None:
            self.network = network
            self.security = security
            self.tcpSettings = tcpSettings
            self.kcpSettings = kcpSettings
            self.wsSettings = wsSettings
            self.httpSettings = httpSettings
            self.tlsSettings = tlsSettings
            self.quicSettings = quicSettings
            self.realitySettings = realitySettings
            self.grpcSettings = grpcSettings
            self.dsSettings = dsSettings
            self.sockopt = sockopt

        def populateTransportSettings(
            self,
            transport: str,
            headerType: str,
            host: str,
            path: str,
            seed: str,
            quicSecurity: str,
            key: str,
            mode: str,
            serviceName: str,
        ) -> str:
            sni = ""
            self.network = transport
            if self.network == "tcp":
                tcpSetting = self.TcpSettingsBean()
                if headerType == HTTP:
                    tcpSetting.header.type = HTTP
                    if host != "" or path != "":
                        requestObj = self.TcpSettingsBean.HeaderBean.RequestBean()
                        requestObj.headers.Host = (
                            "" if host == None else host.split(",")
                        )
                        requestObj.path = "" if path == None else path.split(",")
                        tcpSetting.header.request = requestObj
                        sni = (
                            requestObj.headers.Host[0]
                            if len(requestObj.headers.Host) > 0
                            else sni
                        )
                else:
                    tcpSetting.header.type = "none"
                    sni = host if host != "" else ""
                self.tcpSetting = tcpSetting

            elif self.network == "kcp":
                kcpsetting = self.KcpSettingsBean()
                kcpsetting.header.type = headerType if headerType != None else "none"
                if seed == None or seed == "":
                    kcpsetting.seed = None
                else:
                    kcpsetting.seed = seed
                self.kcpSettings = kcpsetting

            elif self.network == "ws":
                wssetting = self.WsSettingsBean()
                wssetting.headers.Host = host if host != None else ""
                sni = wssetting.headers.Host
                wssetting.path = path if path != None else "/"
                self.wsSettings = wssetting

            elif self.network == "h2" or self.network == "http":
                network = "h2"
                h2Setting = self.HttpSettingsBean()
                h2Setting.host = "" if host == None else host.split(",")
                sni = h2Setting.host[0] if len(h2Setting.host) > 0 else sni
                h2Setting.path = path if path != None else "/"
                self.httpSettings = h2Setting

            elif self.network == "quic":
                quicsetting = self.QuicSettingBean()
                quicsetting.security = quicSecurity if quicSecurity != None else "none"
                quicsetting.key = key if key != None else ""
                quicsetting.header.type = headerType if headerType != None else "none"
                self.quicSettings = quicsetting

            elif self.network == "grpc":
                grpcSetting = self.GrpcSettingsBean()
                grpcSetting.multiMode = mode == "multi"
                grpcSetting.serviceName = serviceName if serviceName != None else ""
                sni = host if host != None else ""
                self.grpcSettings = grpcSetting

            return sni

        def populateTlsSettings(
            self,
            streamSecurity: str,
            allowInsecure: bool,
            sni: str,
            fingerprint: str,
            alpns: str,
            publicKey: str,
            shortId: str,
            spiderX: str,
        ):
            self.security = streamSecurity
            tlsSetting = self.TlsSettingsBean(
                allowInsecure=allowInsecure,
                serverName=sni,
                fingerprint=fingerprint,
                alpn=None if alpns == None or alpns == "" else alpns.split(","),
                publicKey=publicKey,
                shortId=shortId,
                spiderX=spiderX,
            )

            if self.security == TLS:
                self.tlsSettings = tlsSetting
                self.realitySettings = None
            elif self.security == REALITY:
                self.tlsSettings = None
                self.realitySettings = tlsSetting

    class MuxBean:
        enabled: bool
        concurrency: int

        def __init__(self, enabled: bool, concurrency: int = 8):
            self.enabled = enabled
            self.concurrency = concurrency

    tag: str = "proxy"
    protocol: str
    settings: OutSettingsBean = None
    streamSettings: StreamSettingsBean = None
    proxySettings: any = None
    sendThrough: str = None
    mux: MuxBean = MuxBean(False)

    def __init__(
        self,
        tag: str = "proxy",
        protocol: str = None,
        settings: OutSettingsBean = None,
        streamSettings: StreamSettingsBean = None,
        proxySettings: any = None,
        sendThrough: str = None,
        mux: MuxBean = MuxBean(enabled=False),
    ):
        self.tag = tag
        self.protocol = protocol
        self.settings = settings
        self.streamSettings = streamSettings
        self.proxySettings = proxySettings
        self.sendThrough = sendThrough
        self.mux = mux


class DnsBean:
    class ServersBean:
        address: str = ""
        port: int = None
        domains: list[str] = None  # str
        expectIPs: list[str] = None  # str
        clientIp: str = None

        def __init__(
            self,
            address: str = "",
            port: int = None,
            domains: list[str] = None,
            expectIPs: list[str] = None,
            clientIp: str = None,
        ) -> None:
            self.address = address
            self.port = port
            self.domains = domains
            self.expectIPs = expectIPs
            self.clientIp = clientIp

    servers: list[any] = None  # any
    hosts: list = None  # map(str, any)
    clientIp: str = None
    disableCache: bool = None
    queryStrategy: str = None
    tag: str = None

    def __init__(
        self,
        servers: list[any] = None,
        hosts: list = None,
        clientIp: str = None,
        disableCache: bool = None,
        queryStrategy: str = None,
        tag: str = None,
    ) -> None:
        self.servers = servers
        self.hosts = hosts
        self.clientIp = clientIp
        self.disableCache = disableCache
        self.queryStrategy = queryStrategy
        self.tag = tag


class RoutingBean:
    class RulesBean:
        type: str = ""
        ip: list[str] = None  # str
        domain: list[str] = None  # str
        outboundTag: str = ""
        balancerTag: str = None
        port: str = None
        sourcePort: str = None
        network: str = None
        source: list[str] = None  # str
        user: list[str] = None  # str
        inboundTag: list[str] = None  # str
        protocol: list[str] = None  # str
        attrs: str = None
        domainMatcher: str = None

        def __init__(
            self,
            type: str = "",
            ip: list[str] = None,
            domain: list[str] = None,
            outboundTag: str = "",
            balancerTag: str = None,
            port: str = None,
            sourcePort: str = None,
            network: str = None,
            source: list[str] = None,
            user: list[str] = None,
            inboundTag: list[str] = None,
            protocol: list[str] = None,
            attrs: str = None,
            domainMatcher: str = None,
        ) -> None:
            self.type = type
            self.ip = ip
            self.domain = domain
            self.outboundTag = outboundTag
            self.balancerTag = balancerTag
            self.port = port
            self.sourcePort = sourcePort
            self.network = network
            self.source = source
            self.user = user
            self.inboundTag = inboundTag
            self.protocol = protocol
            self.attrs = attrs
            self.domainMatcher = domainMatcher

    domainStrategy: str
    domainMatcher: str = None
    rules: list[RulesBean]  # RulesBean
    balancers: list[any]  # any

    def __init__(
        self,
        domainStrategy: str,
        domainMatcher: str = None,
        rules: list[RulesBean] = [],
        balancers: list[any] = [],
    ) -> None:
        self.domainStrategy = domainStrategy
        self.domainMatcher = domainMatcher
        self.rules = rules
        self.balancers = balancers


class FakednsBean:
    ipPool: str = "198.18.0.0/15"
    poolSize: int = 10000

    def __init__(self, ipPool: str = "198.18.0.0/15", poolSize: int = 10000) -> None:
        self.ipPool = ipPool
        self.poolSize = poolSize


class PolicyBean:
    class LevelBean:
        handshake: int = None
        connIdle: int = None
        uplinkOnly: int = None
        downlinkOnly: int = None
        statsUserUplink: bool = None
        statsUserDownlink: bool = None
        bufferSize: int = None

        def __init__(
            self,
            handshake: int = None,
            connIdle: int = None,
            uplinkOnly: int = None,
            downlinkOnly: int = None,
            statsUserUplink: bool = None,
            statsUserDownlink: bool = None,
            bufferSize: int = None,
        ) -> None:
            self.handshake = handshake
            self.connIdle = connIdle
            self.uplinkOnly = uplinkOnly
            self.downlinkOnly = downlinkOnly
            self.statsUserUplink = statsUserUplink
            self.statsUserDownlink = statsUserDownlink
            self.bufferSize = bufferSize

    levels: list  # map(str, LevelBean)
    system: any = None

    def __init__(self, levels: list, system: any = None) -> None:
        self.levels = levels
        self.system = system


class Comment:
    remark: str = None

    def __init__(self, remark: str = None) -> None:
        self.remark = remark


class V2rayConfig:
    _comment: Comment = None
    stats: any = None
    log: LogBean
    policy: PolicyBean
    inbounds: list[InboundBean]  # InboundBean
    outbounds: list[OutboundBean]  # OutboundBean
    dns: DnsBean
    routing: RoutingBean
    api: any = None
    transport: any = None
    reverse: any = None
    fakedns: any = None
    browserForwarder: any = None

    def __init__(
        self,
        _comment: Comment = None,
        stats: any = None,
        log: LogBean = None,
        policy: PolicyBean = None,
        inbounds: list = None,
        outbounds: list = None,
        dns: DnsBean = None,
        routing: RoutingBean = None,
        api: any = None,
        transport: any = None,
        reverse: any = None,
        fakedns: any = None,
        browserForwarder: any = None,
    ) -> None:
        self.stats = stats
        self._comment = _comment
        self.log = log
        self.policy = policy
        self.inbounds = inbounds
        self.outbounds = outbounds
        self.dns = dns
        self.routing = routing
        self.api = api
        self.transport = transport
        self.reverse = reverse
        self.fakedns = fakedns
        self.browserForwarder = browserForwarder


class VmessQRCode:
    v: str = ""
    ps: str = ""
    add: str = ""
    port: str = ""
    id: str = ""
    aid: str = "0"
    scy: str = ""
    net: str = ""
    type: str = ""
    host: str = ""
    path: str = ""
    tls: str = ""
    sni: str = ""
    alpn: str = ""
    allowInsecure: str = ""

    def __init__(
        self,
        v: str = "",
        ps: str = "",
        add: str = "",
        port: str = "",
        id: str = "",
        aid: str = "0",
        scy: str = "",
        net: str = "",
        type: str = "",
        host: str = "",
        path: str = "",
        tls: str = "",
        sni: str = "",
        alpn: str = "",
        allowInsecure: str = "",
        fp: str = "",
    ):
        self.v = v
        self.ps = ps
        self.add = add
        self.port = port
        self.id = id
        self.aid = aid
        self.scy = scy
        self.net = net
        self.type = type
        self.host = host
        self.path = path
        self.tls = tls
        self.sni = sni
        self.alpn = alpn
        self.allowInsecure = allowInsecure
        self.fp = fp


def remove_nulls(d):
    if isinstance(d, dict):
        for k, v in list(d.items()):
            if v is None:
                del d[k]
            else:
                remove_nulls(v)
    if isinstance(d, list):
        for v in d:
            remove_nulls(v)
    return d


def get_log():
    log = LogBean(access="", error="", loglevel="error", dnsLog=False)
    return log


def get_inbound():
    inbound = InboundBean(
        tag="in_proxy",
        port=1080,
        protocol=EConfigType.SOCKS.protocolName,
        listen="127.0.0.1",
        settings=InboundBean.InSettingsBean(
            auth="noauth",
            udp=True,
            userLevel=8,
        ),
        sniffing=InboundBean.SniffingBean(
            enabled=False,
            destOverride=None,
            metadataOnly=None,
        ),
        streamSettings=None,
        allocate=None,
    )
    return inbound


def get_outbound_vmess():
    outbound = OutboundBean(
        protocol=EConfigType.VMESS.protocolName,
        settings=OutboundBean.OutSettingsBean(
            vnext=[
                OutboundBean.OutSettingsBean.VnextBean(
                    users=[OutboundBean.OutSettingsBean.VnextBean.UsersBean()],
                ),
            ]
        ),
        streamSettings=OutboundBean.StreamSettingsBean(),
    )
    return outbound


def get_outbound_vless():
    outbound = OutboundBean(
        protocol=EConfigType.VLESS.protocolName,
        settings=OutboundBean.OutSettingsBean(
            vnext=[
                OutboundBean.OutSettingsBean.VnextBean(
                    users=[OutboundBean.OutSettingsBean.VnextBean.UsersBean()],
                ),
            ]
        ),
        streamSettings=OutboundBean.StreamSettingsBean(),
    )
    return outbound


def get_outbound_trojan():
    outbound = OutboundBean(
        protocol=EConfigType.TROJAN.protocolName,
        settings=OutboundBean.OutSettingsBean(
            servers=[OutboundBean.OutSettingsBean.ServersBean()]
        ),
        streamSettings=OutboundBean.StreamSettingsBean(),
    )
    return outbound


def get_outbound_ss():
    outbound = OutboundBean(
        protocol="shadowsocks",
        settings=OutboundBean.OutSettingsBean(
            servers=[OutboundBean.OutSettingsBean.ServersBean()]
        ),
        streamSettings=OutboundBean.StreamSettingsBean(),
    )
    return outbound


def try_resolve_resolve_sip002(str: str, config: OutboundBean):
    try:
        uri = urlparse(str)
        config.remarks = unquote(uri.fragment or "")

        if ":" in uri.username:
            arr_user_info = list(map(str.strip, uri.username.split(":")))
            if len(arr_user_info) != 2:
                return False
            method = arr_user_info[0]
            password = unquote(arr_user_info[1])
        else:
            base64_decode = base64.b64decode(uri.username).decode(
                encoding="utf-8", errors="ignore"
            )
            arr_user_info = list(map(str.strip, base64_decode.split(":")))
            if len(arr_user_info) < 2:
                return False
            method = arr_user_info[0]
            password = base64_decode.split(":", 1)[1]

        server = config.outbound_bean.settings.servers[0]
        server.address = uri.hostname
        server.port = uri.port
        server.password = password
        server.method = method

        return True
    except Exception as e:
        return False


def get_outbound1():
    outbound1 = OutboundBean(
        tag="direct",
        protocol=EConfigType.FREEDOM.protocolName,
        settings=OutboundBean.OutSettingsBean(
            domainStrategy=DomainStrategy.UseIp,
        ),
        mux=None,
    )
    return outbound1


def get_outbound2():
    outbound2 = OutboundBean(
        tag="blackhole",
        protocol=EConfigType.BLACKHOLE.protocolName,
        settings=OutboundBean.OutSettingsBean(),
        mux=None,
    )
    return outbound2


def get_dns(dns_list=["8.8.8.8"]):
    if isinstance(dns_list, str):
        if "," in dns_list:
            dns_list = dns_list.split(",")

    dns = DnsBean(servers=dns_list)
    return dns


def get_routing():
    routing = RoutingBean(domainStrategy=DomainStrategy.UseIp)
    return routing


def generateConfig(config: str, dns_list=["8.8.8.8"]):
    allowInsecure = True

    temp = config.split("://")
    protocol = temp[0]
    raw_config = temp[1]

    if protocol == EConfigType.VMESS.protocolName:
        _len = len(raw_config)
        if _len % 4 > 0:
            raw_config += "=" * (4 - _len % 4)

        b64decode = base64.b64decode(raw_config).decode(
            encoding="utf-8", errors="ignore"
        )
        _json = json.loads(b64decode, strict=False)

        vmessQRCode_attributes = list(VmessQRCode.__dict__["__annotations__"].keys())
        for key in list(_json.keys()):
            if key not in vmessQRCode_attributes:
                del _json[key]

        vmessQRCode = VmessQRCode(**_json)

        outbound = get_outbound_vmess()

        vnext = outbound.settings.vnext[0]
        vnext.address = vmessQRCode.add
        vnext.port = int(vmessQRCode.port)

        user = vnext.users[0]
        user.id = vmessQRCode.id
        user.security = vmessQRCode.scy if vmessQRCode.scy != "" else DEFAULT_SECURITY
        user.alterId = int(vmessQRCode.aid)

        streamSetting = outbound.streamSettings

        sni = streamSetting.populateTransportSettings(
            transport=vmessQRCode.net,
            headerType=vmessQRCode.type,
            host=vmessQRCode.host,
            path=vmessQRCode.path,
            seed=vmessQRCode.path,
            quicSecurity=vmessQRCode.host,
            key=vmessQRCode.path,
            mode=vmessQRCode.type,
            serviceName=vmessQRCode.path,
        )

        fingerprint = (
            vmessQRCode.fp
            if vmessQRCode.fp
            else streamSetting.tlsSettings.fingerprint
            if streamSetting.tlsSettings
            else None
        )
        try:
            parsed_url = urlparse(config)
            netquery = dict(
                (k, v if len(v) > 1 else v[0])
                for k, v in parse_qs(parsed_url.query).items()
            )
        except:
            pass
        streamSetting.populateTlsSettings(
            streamSecurity=vmessQRCode.tls,
            allowInsecure=allowInsecure,
            sni=sni if vmessQRCode.sni == "" else vmessQRCode.sni,
            fingerprint=netquery.get("fp", None),
            alpns=vmessQRCode.alpn,
            publicKey=netquery.get("pbk", ""),
            shortId=netquery.get("sid", ""),
            spiderX=netquery.get("spx", ""),
        )

        v2rayConfig = V2rayConfig(
            _comment=Comment(remark=vmessQRCode.ps),
            log=get_log(),
            inbounds=[get_inbound()],
            outbounds=[outbound, get_outbound1(), get_outbound2()],
            dns=get_dns(dns_list=dns_list),
            routing=get_routing(),
        )

        v2rayConfig_str_json = json.dumps(v2rayConfig, default=vars)

        res = json.loads(v2rayConfig_str_json)
        res = remove_nulls(res)

        return json.dumps(res)

    elif protocol == EConfigType.VLESS.protocolName:
        parsed_url = urlparse(config)
        _netloc = parsed_url.netloc.split("@")

        name = parsed_url.fragment
        hostname = _netloc[1].split(":")[0]
        port = int(_netloc[1].split(":")[1])
        uid = _netloc[0]

        netquery = dict(
            (k, v if len(v) > 1 else v[0])
            for k, v in parse_qs(parsed_url.query).items()
        )

        outbound = get_outbound_vless()

        streamSetting = outbound.streamSettings
        fingerprint = (
            streamSetting.tlsSettings.fingerprint
            if streamSetting.tlsSettings != None
            else None
        )

        vnext = outbound.settings.vnext[0]
        vnext.address = hostname
        vnext.port = port

        user = vnext.users[0]
        user.id = uid
        user.encryption = netquery.get("encryption", "none")
        user.flow = netquery.get("flow", "")

        sni = streamSetting.populateTransportSettings(
            transport=netquery.get("type", "tcp"),
            headerType=netquery.get("headerType", None),
            host=netquery.get("host", None),
            path=netquery.get("path", None),
            seed=netquery.get("seed", None),
            quicSecurity=netquery.get("quicSecurity", None),
            key=netquery.get("key", None),
            mode=netquery.get("mode", None),
            serviceName=netquery.get("serviceName", None),
        )
        streamSetting.populateTlsSettings(
            streamSecurity=netquery.get("security", ""),
            allowInsecure=allowInsecure,
            sni=sni if netquery.get("sni", None) == None else netquery.get("sni", None),
            fingerprint=netquery.get("fp", fingerprint),
            alpns=netquery.get("alpn", None),
            publicKey=netquery.get("pbk", ""),
            shortId=netquery.get("sid", ""),
            spiderX=netquery.get("spx", ""),
        )

        v2rayConfig = V2rayConfig(
            _comment=Comment(remark=name),
            log=get_log(),
            inbounds=[get_inbound()],
            outbounds=[outbound, get_outbound1(), get_outbound2()],
            dns=get_dns(dns_list=dns_list),
            routing=get_routing(),
        )

        v2rayConfig_str_json = json.dumps(v2rayConfig, default=vars)

        res = json.loads(v2rayConfig_str_json)
        res = remove_nulls(res)

        return json.dumps(res)

    elif protocol == EConfigType.TROJAN.protocolName:
        parsed_url = urlparse(config)
        _netloc = parsed_url.netloc.split("@")

        name = parsed_url.fragment
        hostname = _netloc[1].split(":")[0]
        port = int(_netloc[1].split(":")[1])
        uid = _netloc[0]

        netquery = dict(
            (k, v if len(v) > 1 else v[0])
            for k, v in parse_qs(parsed_url.query).items()
        )

        outbound = get_outbound_trojan()

        streamSetting = outbound.streamSettings

        flow = ""
        fingerprint = (
            streamSetting.tlsSettings.fingerprint
            if streamSetting.tlsSettings != None
            else Fingerprint.randomized
        )

        if len(netquery) > 0:
            sni = streamSetting.populateTransportSettings(
                transport=netquery.get("type", "tcp"),
                headerType=netquery.get("headerType", None),
                host=netquery.get("host", None),
                path=netquery.get("path", None),
                seed=netquery.get("seed", None),
                quicSecurity=netquery.get("quicSecurity", None),
                key=netquery.get("key", None),
                mode=netquery.get("mode", None),
                serviceName=netquery.get("serviceName", None),
            )

            streamSetting.populateTlsSettings(
                streamSecurity=netquery.get("security", TLS),
                allowInsecure=allowInsecure,
                sni=sni
                if netquery.get("sni", None) == None
                else netquery.get("sni", None),
                fingerprint=netquery.get("fp", fingerprint),
                alpns=netquery.get("alpn", None),
                publicKey=netquery.get("pbk", ""),
                shortId=netquery.get("sid", ""),
                spiderX=netquery.get("spx", ""),
            )

            flow = netquery.get("flow", "")

        else:
            streamSetting.populateTlsSettings(
                streamSecurity=TLS,
                allowInsecure=allowInsecure,
                sni="",
                fingerprint=netquery.get("fp", fingerprint),
                alpns=None,
                publicKey=None,
                shortId=None,
                spiderX=None,
            )

        server = outbound.settings.servers[0]
        server.address = hostname
        server.port = port
        server.password = uid
        server.flow = flow

        v2rayConfig = V2rayConfig(
            _comment=Comment(remark=name),
            log=get_log(),
            inbounds=[get_inbound()],
            outbounds=[outbound, get_outbound1(), get_outbound2()],
            dns=get_dns(dns_list=dns_list),
            routing=get_routing(),
        )

        v2rayConfig_str_json = json.dumps(v2rayConfig, default=vars)

        res = json.loads(v2rayConfig_str_json)
        res = remove_nulls(res)

        return json.dumps(res)

    elif protocol == EConfigType.SHADOWSOCKS.protocolName:
        outbound = get_outbound_ss()
        if not try_resolve_resolve_sip002(raw_config, outbound):
            result = raw_config.replace(EConfigType.SHADOWSOCKS.protocolScheme, "")
            index_split = result.find("#")
            if index_split > 0:
                try:
                    outbound.remarks = unquote(result[index_split + 1 :])
                except Exception as e:
                    None  # print(e)

                result = result[:index_split]

            # part decode
            index_s = result.find("@")
            result = (
                base64.b64decode(result[:index_s]).decode(
                    encoding="utf-8", errors="ignore"
                )
                + result[index_s:]
                if index_s > 0
                else base64.b64decode(result).decode(encoding="utf-8", errors="ignore")
            )

            legacy_pattern = re.compile(r"^(.+?):(.*)@(.+):(\d+)\/?.*$")
            match = legacy_pattern.match(result)

            if not match:
                raise Exception("Incorrect protocol")

            server = outbound.settings.servers[0]
            server.address = match.group(3).strip("[]")
            server.port = int(match.group(4))
            server.password = match.group(2)
            server.method = match.group(1).lower()

            v2rayConfig = V2rayConfig(
                _comment=Comment(remark=outbound.remarks),
                log=get_log(),
                inbounds=[get_inbound()],
                outbounds=[outbound, get_outbound1(), get_outbound2()],
                dns=get_dns(dns_list=dns_list),
                routing=get_routing(),
            )

            v2rayConfig_str_json = json.dumps(v2rayConfig, default=vars)

        res = json.loads(v2rayConfig_str_json)
        res = remove_nulls(res)

        return json.dumps(res)


def generate_v2rayconfig_with_name(response):
    json_out = json.loads(generateConfig(response))
    name = json_out["_comment"]["remark"]
    if not name:
        a = json_out["outbounds"][0]["protocol"]
        name = f"{a}_{generate_random_alphanumeric()}"
        json_out["_comment"]["remark"] = name
    return json_out, name


if __name__ == "__main__":
    pass
    # parser = argparse.ArgumentParser(
    #     description="v2ray2json convert vmess, vless, trojan, ... link to client json config."
    # )
    # parser.add_argument(
    #     "config",
    #     nargs="?",
    #     help="A vmess://, vless://, trojan://, ... link.",
    # )

    # option = parser.parse_args()
    config = r"trojan://LjWyuE2d8M@hels.ddns.net:44838?security=reality&sni=yahoo.com&fp=firefox&pbk=8LIuGGsdhR59qjyRALAmGKNuKVlyH3t8OqJmRRdyKl4&sid=92cd56b1&spx=%2F&type=grpc"
    jssss, name = generate_v2rayconfig_with_name(config)
    print(jssss)



import json
import os

from libs.in_win import config_path, inside_windows


class SaveGUIConfigPage:
    def __init__(self, filename, page_data, new_file: bool = False, group="") -> None:
        self.remark = filename
        self.page_data = page_data

        if len(group) > 1:
            self.group_path = f"\\subscriptions\\{group}"
        else:
            self.group_path = ""

        data: dict = self.read_content_of_config_file_as_json()

        if "protocol" in self.page_data:
            if self.page_data["protocol"] in ("vless", "vmess", "trojan"):
                new_data = self.edit_v2ray_configs(data)
                data.update(new_data)
                self.save_data(data)
        else:
            self.save_data(page_data)

    def edit_v2ray_configs(self, data: json) -> dict:
        data["outbounds"][0]["protocol"] = self.page_data["protocol"]
        data["outbounds"][0]["streamSettings"]["network"] = self.page_data["network"]
        if "trojan" in self.page_data["protocol"]:
            data["outbounds"][0]["settings"]["servers"][0]["address"] = self.page_data[
                "server_address"
            ]
            data["outbounds"][0]["settings"]["servers"][0]["method"] = self.page_data[
                "method"
            ]
            data["outbounds"][0]["settings"]["servers"][0]["ota"] = self.page_data[
                "ota"
            ]
            data["outbounds"][0]["settings"]["servers"][0]["password"] = self.page_data[
                "password"
            ]
            data["outbounds"][0]["settings"]["servers"][0]["port"] = self.page_data[
                "port"
            ]
        elif ("vless" in self.page_data["protocol"]) or (
            "vmess" in self.page_data["protocol"]
        ):
            data["outbounds"][0]["settings"]["vnext"][0]["address"] = self.page_data[
                "server_address"
            ]
            data["outbounds"][0]["settings"]["vnext"][0]["port"] = self.page_data[
                "port"
            ]
            data["outbounds"][0]["settings"]["vnext"][0]["users"][0][
                "security"
            ] = self.page_data["security"]

            data["outbounds"][0]["settings"]["vnext"][0]["users"][0][
                "id"
            ] = self.page_data["uuid"]

        if self.page_data["network"] == "grpc":
            try:
                data["outbounds"][0]["streamSettings"]["grpcSettings"][
                    "serviceName"
                ] = self.page_data["grpc_serviceName"]
            except:
                pass

        try:
            data["outbounds"][0]["streamSettings"]["security"] = self.page_data[
                "security"
            ]
            if self.page_data["security"] == "reality":
                data["outbounds"][0]["streamSettings"]["realitySettings"][
                    "serverName"
                ] = self.page_data["serverName"]
                data["outbounds"][0]["streamSettings"]["realitySettings"][
                    "fingerprint"
                ] = self.page_data["fingerprint"]
                data["outbounds"][0]["streamSettings"]["realitySettings"][
                    "publicKey"
                ] = self.page_data["publicKey"]
                data["outbounds"][0]["streamSettings"]["realitySettings"][
                    "shortId"
                ] = self.page_data["shortId"]
                data["outbounds"][0]["streamSettings"]["realitySettings"][
                    "spiderX"
                ] = self.page_data["spiderX"]
            # elif self.security == "reality":
            elif self.page_data["security"] == "tls":
                data["outbounds"][0]["streamSettings"]["tlsSettings"][
                    "serverName"
                ] = self.page_data["serverName"]
                data["outbounds"][0]["streamSettings"]["tlsSettings"][
                    "fingerprint"
                ] = self.page_data["fingerprint"]
                data["outbounds"][0]["streamSettings"]["tlsSettings"]["alpn"] = (
                    str(self.page_data["alpn"]).replace("[", "").replace("]", "")
                )

        except:
            pass

        return data

    def read_content_of_config_file_as_json(self) -> json:
        if "protocol" in self.page_data:
            if self.page_data["protocol"] in ("vless", "vmess", "trojan"):
                json_config_path = (
                    f"{config_path()}\\v2ray_profiles{self.group_path}\\{self.remark}"
                )
        else:
            json_config_path = (
                f"{config_path()}\\gost_profiles{self.group_path}\\{self.remark}"
            )

        if not inside_windows():
            json_config_path = json_config_path.replace("\\", "/")

        with open(json_config_path, "r") as json_file:
            # Load the JSON data
            data = json.load(json_file)
        return data

    def create_path_if_not_exists(self, path):
        if not os.path.exists(path):
            try:
                os.makedirs(path)  # This will create directories recursively
                print(f"Path '{path}' did not exist. Created successfully.")
            except OSError as e:
                print(f"Error creating path '{path}': {e}")
        else:
            print(f"Path '{path}' already exists.")

    def save_data(self, data) -> json:
        if "protocol" in self.page_data:
            if self.page_data["protocol"] in ("vless", "vmess", "trojan"):
                path = f"{config_path()}\\v2ray_profiles{self.group_path}\\"
                if not inside_windows():
                    path = path.replace("\\", "/")
                json_config_path = f"{path}{self.remark}"
        else:
            path = f"{config_path()}\\gost_profiles{self.group_path}\\"
            if not inside_windows():
                path = path.replace("\\", "/")

            json_config_path = f"{path}{self.remark}"

        self.create_path_if_not_exists(path)

        with open(json_config_path, "w") as json_file:
            # Load the JSON data
            json.dump(data, json_file)



import base64
import hashlib
import json
import os
from urllib.parse import parse_qs, urlencode, urlparse

from libs.in_win import config_path


class ConfigDataScheme:
    protocol = ""
    uuid = ""
    address = ""
    port = ""
    attributes = ""


class URLTools_for_V2Ray:
    def __init__(self, url: str, localport=7595) -> None:
        self.localport = localport
        self.url2Config(url)
        self.mainConfigGenerator()

    def toJsonConfigString(self):
        return self.finalConfig

    def containsBase64(self, text):
        try:
            base64.b64decode(text).decode("utf-8")
        except:
            # print(f"not base64: {e}")
            return False

        return True

    # def v2ray_json_to_url(self, json_data: dict):
    #     #       self.page_data["remark"] = self.window["alias"].get()
    #     # self.page_data["server_address"] = self.window["address"].get()
    #     # self.page_data["port"] = self.window["port"].get()
    #     # self.page_data["uuid"] = self.window["uuid"].get()
    #     # self.page_data["user_security"] = self.window["encryptionmethod"].get()
    #     # # self.page_data["grpc_serviceName"] = "imi"
    #     # self.page_data["network"] = self.window["transport_protocol"].get()
    #     # self.page_data["security"] = self.window["tls"].get()
    #     # self.page_data["sni"] = self.page_data["serverName"] = self.window["sni"].get()
    #     # self.page_data["fingerprint"] = self.window["fingerprint"].get()
    #     # self.page_data["publicKey"] = self.window["publickey"].get()
    #     # self.page_data["shortId"] = self.window["shortid"].get()
    #     # self.page_data["spiderX"] = self.window["spiderx"].get()

    #     attributes = {
    #         "v": "2",
    #         "ps": json_data["remark"],
    #         "add": json_data["server_address"],
    #         "port": json_data["port"],
    #         "id": json_data["uuid"],
    #         "aid": json_data["aid"],
    #         "type": json_data["network"],
    #         "security": json_data["security"],
    #         "spx": json_data["spiderX"],
    #         "sid": json_data["shortId"],
    #         "pbk": json_data["publicKey"],
    #         "fp": json_data["fingerprint"],
    #         "sni": json_data["sni"],
    #     }
    #     # base64_config = base64.urlsafe_b64encode(json.dumps(attributes).encode("utf-8")).decode("utf-8")

    #     # Create the V2Ray VMess URL
    #     v2ray_url = f"{json_data['protocol']}://"

    def generate_random_md5_hash(self):
        return hashlib.md5(os.urandom(128)).hexdigest()

    def url2Config(self, text: str):
        split_parts = text.split("://")
        protocol = split_parts[0]
        text1 = split_parts[1]
        self.configData = ConfigDataScheme()
        if self.containsBase64(text1):
            self.configData.protocol = protocol
            decoded_text = base64.b64decode(text1).decode("utf-8")
            self.configData.attributes = json.loads(decoded_text)
            self.configData.protocol = protocol
            self.configData.uuid = self.configData.attributes["id"]
            self.configData.address = self.configData.attributes["add"]
            self.configData.port = self.configData.attributes["port"]
            try:
                self.profileName = self.configData.attributes["ps"]
            except:
                self.profileName = f"{self.configData.protocol}_{self.configData.attributes['type']}_{self.generate_random_md5_hash()}"

        else:
            parsed_url = urlparse(text)
            # Extracting data from the URL
            self.configData.protocol = parsed_url.scheme
            self.configData.uuid = parsed_url.username
            self.configData.address = parsed_url.hostname
            self.configData.port = parsed_url.port
            query_params = parse_qs(parsed_url.query)
            self.configData.attributes = {
                key: value[0] for key, value in query_params.items()
            }

            try:
                self.profileName = text.split("#")[1].strip()
            except:
                self.profileName = f"{self.configData.protocol}_{self.configData.attributes['type']}_{self.generate_random_md5_hash()}"

    def generate_scheme_settings(self):
        if self.configData.protocol in ["vmess", "vless"]:
            config_template = r"""{
            "protocol": "PROTOCOL",
            "settings": {
            "vnext": [
                {
                    "address": "ADDR",
                    "port": PORT,
                    "users": [
                    {
                        "id": "ID",
                        "alterId": 0,
                        "email": "t@t.tt",
                        "security": "auto",
                        "encryption": "none"
                    }
                    ]
                }
                ]}
            }"""

            scheme = (
                config_template.replace("PROTOCOL", self.configData.protocol)
                .replace("ADDR", self.configData.address)
                .replace("PORT", str(self.configData.port))
                .replace("ID", self.configData.uuid)
            )
            self.scheme = json.loads(scheme)
        elif self.configData.protocol == "trojan":
            config_template = r"""{
                "protocol": "PROTOCOL",
                "settings": {
                    "servers": [
                    {
                        "address": "ADDR",
                        "method": "chacha20",
                        "ota": false,
                        "password": "PAASWORD",
                        "port": PORT,
                        "level": 1
                    }
                    ]
                }
            }
            """
            scheme = (
                config_template.replace("PROTOCOL", self.configData.protocol)
                .replace("ADDR", self.configData.address)
                .replace("PORT", str(self.configData.port))
                .replace("PAASWORD", self.configData.uuid)
            )
            self.scheme = json.loads(scheme)

        # return json.dumps(self.configData.attributes, indent=4)

    def generate_streamSettings(self):
        attributes = self.configData.attributes
        if "reality" in str(attributes):
            if attributes["type"] == "tcp":
                realityconfig = r"""{
                "streamSettings": {
                    "network": "tcp",
                    "security": "reality",
                    "realitySettings": {
                        "serverName": "sni",
                        "fingerprint": "fp",
                        "show": false,
                        "publicKey": "pbk",
                        "shortId": "sid",
                        "spiderX": "spx"
                    }
                }
                }
                """
                realityconfig = (
                    realityconfig.replace("sni", attributes["sni"])
                    .replace("fp", attributes["fp"])
                    .replace("pbk", attributes["pbk"])
                    .replace("spx", attributes["spx"])
                )

                try:
                    realityconfig = realityconfig.replace("sid", attributes["sid"])
                except:
                    realityconfig = realityconfig.replace("sid", "")

                return json.loads(realityconfig)  # json.dumps(realityconfig, indent=4)

            elif attributes["type"] == "grpc":
                realityconfig = r"""{
                "streamSettings": {
                        "network": "grpc",
                        "security": "reality",
                        "realitySettings": {
                        "serverName": "sni",
                        "fingerprint": "fp",
                        "show": false,
                        "publicKey": "pbk",
                        "shortId": "SID",
                        "spiderX": "spx"
                        },
                        "grpcSettings": {
                        "serviceName": "ServiceName",
                        "multiMode": false,
                        "idle_timeout": 60,
                        "health_check_timeout": 20,
                        "permit_without_stream": false,
                        "initial_windows_size": 0
                        }
                    }
                    }
                    """
                realityconfig = (
                    realityconfig.replace("sni", attributes["sni"])
                    .replace("fp", attributes["fp"])
                    .replace("pbk", attributes["pbk"])
                    .replace("spx", attributes["spx"])
                )

                try:
                    realityconfig = realityconfig.replace("SID", attributes["sid"])
                except:
                    realityconfig = realityconfig.replace("SID", "")

                try:
                    realityconfig = realityconfig.replace(
                        "ServiceName", attributes["serviceName"]
                    )
                except:
                    realityconfig = realityconfig.replace("ServiceName", "")

                return json.loads(realityconfig)  # json.dumps(realityconfig, indent=4)
        elif "tls" in attributes or "security" in attributes:
            try:
                security = attributes["tls"]
            except:
                security = attributes["security"]
            if security == "tls":
                realityconfig = r"""{
            "streamSettings": {
                    "network": "NET",
                    "security": "tls",
                    "tlsSettings": {
                    "allowInsecure": false,
                    "serverName": "SNI",
                    "alpn": [""],
                    "fingerprint": "FP",
                    "show": false
                    }
                }
                }
                """
                realityconfig = (
                    realityconfig.replace("SNI", attributes["sni"])
                    .replace("FP", attributes["fp"] if "fp" in attributes else "chrome")
                    .replace(
                        "NET",
                        attributes["net"]
                        if "net" in attributes
                        else attributes["type"],
                    )
                )
                data = json.loads(realityconfig)
                data["streamSettings"]["tlsSettings"]["alpn"] = (
                    attributes["alpn"].split(",")
                    if "alpn" in attributes
                    else "http/1.1"
                )
            else:
                realityconfig = """{
                    "streamSettings": {
                    "network": "NET",
                    "wsSettings": {
                    "path": "/",
                    "headers": {}
                    }
                }
                }
                """
                realityconfig = realityconfig.replace("NET", attributes["net"])
                data = json.loads(realityconfig)
            return data

    def mainConfigGenerator(self):
        config_template = """    {
            "log": {
                "access": "",
                "error": "",
                "loglevel": "debug"
            },
            "inbounds": [
                {
                "tag": "socks",
                "port": LOCALSOCKSPORT,
                "listen": "127.0.0.1",
                "protocol": "socks",
                "sniffing": {
                    "enabled": true,
                    "destOverride": [
                    "http",
                    "tls"
                    ],
                    "routeOnly": true
                },
                "settings": {
                    "auth": "noauth",
                    "udp": true,
                    "allowTransparent": false
                }
                },
                {
                "tag": "http",
                "port": LOCALHTTPPORT,
                "listen": "127.0.0.1",
                "protocol": "http",
                "sniffing": {
                    "enabled": true,
                    "destOverride": [
                    "http",
                    "tls"
                    ],
                    "routeOnly": true
                },
                "settings": {
                    "auth": "noauth",
                    "udp": true,
                    "allowTransparent": false
                }
                }
            ],
            "outbounds": [
                {
                "tag": "proxy",
                PART1,
                PART2,
                "mux": {
                    "enabled": false,
                    "concurrency": -1
                }
                },
                {
                "tag": "direct",
                "protocol": "freedom",
                "settings": {}
                },
                {
                "tag": "block",
                "protocol": "blackhole",
                "settings": {
                    "response": {
                    "type": "http"
                    }
                }
                }
            ],
            "dns": {
                "servers": [
                "1.1.1.1",
                "8.8.8.8"
                ]
            },
            "routing": {
                "domainStrategy": "AsIs",
                "rules": [
                {
                    "type": "field",
                    "inboundTag": [
                    "api"
                    ],
                    "outboundTag": "api"
                },
                {
                    "type": "field",
                    "port": "0-65535",
                    "outboundTag": "proxy"
                }
                ]
            }
            }"""
        self.generate_scheme_settings()
        self.streamSettings = self.generate_streamSettings()
        scheme = self.delete_braces(json.dumps(self.scheme, indent=4))

        streamSettings = self.delete_braces(json.dumps(self.streamSettings, indent=4))
        self.finalConfig = (
            config_template.replace("PART1", scheme)
            .replace("PART2", streamSettings)
            .replace("LOCALSOCKSPORT", str(self.localport))
            .replace("LOCALHTTPPORT", str(self.localport + 1))
        )

        # config = json.loads(config)

    def delete_braces(self, string):
        """Deletes braces from the beginning and end of a string."""
        start_index = string.find("{")
        end_index = string.rfind("}")
        return string[start_index + 1 : end_index]


if __name__ == "__main__":
    # url = r"trojan://EM2y7NdIQJ@shahryar.ddns.net:13616?security=reality&sni=microsoft.com&fp=firefox&pbk=-6LB0M5afd9i1UpzezmGtDYQg-3GCCXEFgio5j_KXCc&sid=8ccc&spx=%2F&type=tcp&headerType=none#shahryar-trojan1-reality"
    # url = r"trojan://o9qIL2MGtR@shahryar.ddns.net:49030?security=reality&sni=microsoft.com&fp=firefox&pbk=tEwI2UUIGMRW8wrWj8pZEREDRDta9EegrwuhLsDXHgk&spx=%2F&type=grpc&serviceName=shahryar.ddns.net&mode=gun#trojan-grpc-r85l64s3v3"
    # url = r"trojan://LjWyuE2d8M@hels.ddns.net:44838?security=reality&sni=yahoo.com&fp=firefox&pbk=8LIuGGsdhR59qjyRALAmGKNuKVlyH3t8OqJmRRdyKl4&sid=92cd56b1&spx=%2F&type=grpc#reality-test-vfbd9h81"
    # url = r"trojan://o9qIL2MGtR@shahryar.ddns.net:49030?security=reality&sni=microsoft.com&fp=firefox&pbk=tEwI2UUIGMRW8wrWj8pZEREDRDta9EegrwuhLsDXHgk&spx=%2F&type=grpc&serviceName=shahryar.ddns.net&mode=gun#trojan-grpc-r85l64s3v3"
    # url = r"vmess://ew0KICAidiI6ICIyIiwNCiAgInBzIjogInZtZXNzX25vdGxzLXAwdW56aDMiLA0KICAiYWRkIjogImhlbHMuZGRucy5uZXQiLA0KICAicG9ydCI6ICIxMzE0NSIsDQogICJpZCI6ICIzMGRiNDRmNS0yNTVmLTRhYmUtOGEzNC00MDg0OTRhNzZhNDkiLA0KICAiYWlkIjogIjAiLA0KICAic2N5IjogImF1dG8iLA0KICAibmV0IjogIndzIiwNCiAgInR5cGUiOiAibm9uZSIsDQogICJob3N0IjogIiIsDQogICJwYXRoIjogIi8iLA0KICAidGxzIjogIm5vbmUiLA0KICAic25pIjogIiIsDQogICJhbHBuIjogIiINCn0="

    # url = r"trojan://EM2y7NdIQJ@shahryar.ddns.net:13616?security=reality&sni=microsoft.com&fp=firefox&pbk=-6LB0M5afd9i1UpzezmGtDYQg-3GCCXEFgio5j_KXCc&sid=8ccc&spx=%2F&type=tcp&headerType=none#shahryar-trojan1-reality"
    url = r"trojan://LjWyuE2d8M@hels.ddns.net:44838?security=reality&sni=yahoo.com&fp=firefox&pbk=8LIuGGsdhR59qjyRALAmGKNuKVlyH3t8OqJmRRdyKl4&sid=92cd56b1&spx=%2F&type=grpc#reality-test-vfbd9h81"
    print(URLTools_for_V2Ray(url).toJsonConfigString())

####################
###################
#####################



import os
import platform
import winreg

import psutil

# Define the local filename to save data
import requests

if os.name == "nt":
    import winsound


class pass_by_ref:
    value = None


class temp:
    def update(self, a):
        pass


class FactorySetting:
    @staticmethod
    def check_file_or_folder_exists(path):
        return os.path.exists(path)

    @staticmethod
    def delete_config_folder():
        path = f"{config_path()}"
        cmd = f"rmdir /s /q {path}"
        if not inside_windows():
            path = path.replace("\\", "/")
            cmd = f"rm -rf {path}"
        os.popen(cmd).read()

    @staticmethod
    def make_config_folder_ready(folder_path):
        ################################temporary remove all subscriptions

        if not inside_windows():
            folder_path = folder_path.replace("\\", "/")
        cmd = f"mkdir {folder_path}"
        if not inside_windows():
            cmd = f"mkdir -p {folder_path}"
        os.popen(cmd).read()

    gui_config = """
{
    "local_port": "8080",
    "selected_profile_name": "",
    "selected_profile_number": 0,
    "use_fragmentation": false,
    "keep_top": false,
    "close_to_tray": false,
    "auto_connect": false,
    "start_minimized": false,
    "cloudflare_address": "bruce.ns.cloudflare.com",
    "segmentation_timeout": "5",
    "num_of_fragments": "77",
    "subscription": "",
    "beep": true
}"""


tmp = temp()


def download_xray_gost(window, enable_download: pass_by_ref, filename):
    if filename == "xray":
        if platform.system() == "Windows":
            url = "https://github.com/iblockchaincyberchain/v2rayp_bin/raw/main/win/xray.exe"
            filename = "xray.exe"

        elif platform.system() == "Linux":
            url = "https://github.com/iblockchaincyberchain/v2rayp_bin/raw/main/linux/xray"

        elif platform.system() == "Darwin":
            url = (
                "https://github.com/iblockchaincyberchain/v2rayp_bin/raw/main/mac/xray"
            )
    elif filename == "gost":
        if platform.system() == "Windows":
            url = "https://github.com/iblockchaincyberchain/v2rayp_bin/raw/main/win/gost.exe"
            filename = "gost.exe"

        elif platform.system() == "Linux":
            url = "https://github.com/iblockchaincyberchain/v2rayp_bin/raw/main/linux/gost"

        elif platform.system() == "Darwin":
            url = (
                "https://github.com/iblockchaincyberchain/v2rayp_bin/raw/main/mac/gost"
            )

    download_binary(url, filename, window, enable_download)


def download_binary(url, filename, window, enable_download: pass_by_ref):
    cwd = os.getcwd()
    path = f"{config_path()}/bin"
    try:
        os.mkdir(path)
    except:
        pass

    chunk_size = 2048
    resp = requests.get(url, stream=True)
    total = int(resp.headers.get("content-length", 0))

    fname_temp = f"{filename}.tmp"
    sum = 0
    perc = 0

    pbar = window["progressbar2"]
    percentage = window["percentage"]

    with open(f"{path}/{fname_temp}", "wb") as file:
        for data in resp.iter_content(chunk_size=chunk_size):
            if not enable_download.value:
                return
                break
            size = file.write(data)
            sum = sum + size
            # print(
            #     f"downloading: {int(100 * sum / total)}%, downloaded {int(sum/1024)} from {int(total/1024)} KBytes."
            # )
            perc = int(100 * sum / total)
            pbar.update(perc)
            percentage.update(f"Total Percentage is: {perc}%")
    # window.close()
    os.chdir(path)
    if not inside_windows():
        os.popen(f"mv {path}/{filename}.tmp {path}/{filename}").read()
        os.popen(f"chmod +x {path}/{filename}").read()
    else:
        path = path.replace("\\", "/")
        os.popen(f"move {path}\\{filename}.tmp {path}\\{filename}")
    os.chdir(cwd)


def beep():
    if os.name == "nt":
        # winsound.PlaySound("SystemAsterisk", winsound.SND_ALIAS | winsound.SND_ASYNC)
        winsound.PlaySound(
            "assets/sounds/beep.wav", winsound.SND_ALIAS | winsound.SND_ASYNC
        )


def check_process_exists(process_name):
    if inside_windows():
        ps_list = os.popen("tasklist").read()
    else:
        ps_list = os.popen("ps").read()
    if process_name in ps_list:
        return True
    return False

    # for proc in psutil.process_iter(["name"]):
    #     if process_name in proc.info["name"]:
    #         return True
    # return False


def inside_windows():
    inside_window = False
    if os.name == "nt":
        inside_window = True
    return inside_window


def config_path():
    inside_window = False
    if os.name == "nt":
        inside_window = True

    if inside_window:
        config_path = f"{os.getenv('USERPROFILE')}\\appdata\\roaming\\v2rayp\\configs"
    else:
        config_path = f'{os.popen("cd ~;pwd").read().strip()}/Documents/v2rayp/configs'
    return config_path


if __name__ == "__main__":
    print(config_path())
    print("gost exist: ", check_process_exists("gost"))
    # download_xray()
    # a = pass_by_ref()
    # a.value = True

    # def temp(b: pass_by_ref):
    #     b.value = False

    # temp(a)
    # print(a.value)


def set_socks5_proxy(proxy_address, proxy_port):
    # Open the Internet Settings registry key
    reg_key = winreg.OpenKey(
        winreg.HKEY_CURRENT_USER,
        r"Software\Microsoft\Windows\CurrentVersion\Internet Settings",
        0,
        winreg.KEY_WRITE,
    )

    # Enable proxy settings
    winreg.SetValueEx(reg_key, "ProxyEnable", 0, winreg.REG_DWORD, 1)

    # Set the proxy server address
    winreg.SetValueEx(
        reg_key, "ProxyServer", 0, winreg.REG_SZ, f"{proxy_address}:{proxy_port}"
    )

    # Set the proxy type to SOCKS5
    winreg.SetValueEx(reg_key, "ProxyServerType", 0, winreg.REG_DWORD, 5)

    # Enable proxy override
    winreg.SetValueEx(reg_key, "ProxyOverride", 0, winreg.REG_SZ, "<local>")

    # Close the registry key
    winreg.CloseKey(reg_key)

    print("SOCKS5 proxy successfully set.")


def reset_proxy_settings():
    # Open the Internet Settings registry key
    reg_key = winreg.OpenKey(
        winreg.HKEY_CURRENT_USER,
        r"Software\Microsoft\Windows\CurrentVersion\Internet Settings",
        0,
        winreg.KEY_WRITE,
    )

    try:
        # Disable proxy settings
        winreg.SetValueEx(reg_key, "ProxyEnable", 0, winreg.REG_DWORD, 0)
    except:
        pass
    # Delete the proxy server address
    try:
        winreg.DeleteValue(reg_key, "ProxyServer")
    except:
        pass
    # Delete the proxy type
    try:
        winreg.DeleteValue(reg_key, "ProxyServerType")
    except:
        pass
    # Delete the proxy override
    try:
        winreg.DeleteValue(reg_key, "ProxyOverride")
    except:
        pass
    # Close the registry key
    winreg.CloseKey(reg_key)

    print("System proxy settings successfully reset.")



import os
import random
import string
import uuid


def generate_random_alphanumeric(length=5):
    characters = string.ascii_letters + string.digits
    random_string = "".join(random.choice(characters) for _ in range(length))
    return random_string


def generate_random_filename():
    random_uuid = uuid.uuid4()
    filename = str(random_uuid)
    return filename[0:5]


if __name__ == "__main__":
    random_filename = generate_random_filename()
    print("Random Filename:", random_filename[0:5])
    # Generate a random 5-digit alphanumeric string
    random_5_digit_alphanumeric = generate_random_alphanumeric()
    print("Random 5-digit Alphanumeric:", random_5_digit_alphanumeric)



import json

# Read the JSON file
with open("input.json", "r") as file:
    data = json.load(file)

# Extract the required information
address = data["outbounds"][0]["settings"]["servers"][0]["address"]
password = data["outbounds"][0]["settings"]["servers"][0]["password"]
port = data["outbounds"][0]["settings"]["servers"][0]["port"]
security = data["outbounds"][0]["streamSettings"]["security"]
sni = data["outbounds"][0]["streamSettings"]["realitySettings"]["serverName"]
fp = data["outbounds"][0]["streamSettings"]["realitySettings"]["fingerprint"]
pbk = data["outbounds"][0]["streamSettings"]["realitySettings"]["publicKey"]
sid = data["outbounds"][0]["streamSettings"]["realitySettings"]["shortId"]
spx = data["outbounds"][0]["streamSettings"]["realitySettings"]["spiderX"]

# Construct the link
link = f"trojan://{password}@{address}:{port}?security={security}&sni={sni}&fp={fp}&pbk={pbk}&sid={sid}&spx={spx}&type=grpc#reality-trojan-{port}"

# Print the resulting link
print(link)



import json
import sys

import PySimpleGUI as psg

sys.path.append("v2rayp")
from libs.in_win import config_path, inside_windows


class SubscriptionGUI:
    def __init__(self) -> None:
        self.get_subs()
        self.layout = [[self.generate_top_parts()], [self.generate_buttons()]]

    def get_subs(self):
        subscription_guide_path = f"{config_path()}\\gui\\subscription.json"
        if not inside_windows():
            subscription_guide_path = subscription_guide_path.replace("\\", "/")
        try:
            with open(subscription_guide_path, "r") as file:
                self.json_object = json.load(file)
        except:
            pass

    def getLayout(self) -> list:
        return self.layout

    def generate_top_parts(self):
        rows = []

        try:
            len(self.json_object["subscriptions"])
        except:
            layout = [
                [
                    psg.Text(
                        "Configurations",
                        justification="center",
                        font=("Arial Bold", 18),
                    )
                ],
                [],
            ]
            return layout

        for sub in self.json_object["subscriptions"]:
            subscription_name = sub["subscription_name"]
            url = sub["url"]

            row = [
                [psg.HorizontalSeparator()],
                [
                    psg.Text("Sub Name:\t"),
                    psg.InputText(
                        default_text=subscription_name, key=f"sname_{subscription_name}"
                    ),
                ],
                [
                    psg.Text("URL:\t\t"),
                    psg.InputText(default_text=url, key=f"url_{subscription_name}"),
                ],
                [
                    psg.Column(
                        [[psg.Button("Delete", key=f"delete_{subscription_name}")]],
                        justification="center",
                    )
                ],
            ]
            rows.append(row)

        layout = [
            [
                psg.Text(
                    "Configurations", justification="center", font=("Arial Bold", 18)
                )
            ],
            rows,
        ]
        return layout

    def generate_buttons(self):
        layout = [
            psg.Button(key="confirm", button_text="Confirm"),
            psg.Button(key="new", button_text="New"),
            psg.Button(key="cancel", button_text="Cancel"),
        ]
        return layout

    def start_window(self):
        layout = self.getLayout()
        self.window = psg.Window(
            "Subscription Configs",
            layout,
            # size=(600, 790),
            resizable=True,
            no_titlebar=True,
            grab_anywhere=True,
            keep_on_top=True,
        )
        while True:
            event, values = self.window.read()
            # print("event:", event, "values:", values)

            if "confirm" in event:
                page_data = self.save_return()
                break

            elif "new" in event:
                page_data = self.save_return()
                break

            elif "cancel" in event or event == psg.WIN_CLOSED:
                page_data = None
                break
        self.window.close()
        return page_data


if __name__ == "__main__":
    SubscriptionGUI().start_window()



import json
import os
import time

from libs.in_win import config_path, inside_windows

# Set the encoding explicitly to UTF-8
# if os.name == "nt":  # For Windows
#     import sys

#     sys.stdin.reconfigure(encoding="utf-8")
#     sys.stdout.reconfigure(encoding="utf-8")
# else:  # For Unix/Linux
#     import locale

#     locale.setlocale(locale.LC_ALL, "en_US.utf-8")


class RefereshTableContent:
    def __init__(self) -> None:
        self.json_v2ray_configs = self._referesh_v2ray_folder()
        self.json_gost_configs = self._referesh_gost_folder()
        ##############################
        self.v2ray_subscriptions = []

        if inside_windows():
            path = f"{config_path()}\\v2ray_profiles\\subscriptions"
        else:
            path = f"{config_path()}/v2ray_profiles/subscriptions"

        self.check_folder_if_not_make(path)

        self.v2ray_subscriptions = [
            file for file in os.listdir(path) if os.path.isdir(os.path.join(path, file))
        ]

        print(self.v2ray_subscriptions)
        # if inside_windows():
        #     cmd = f"dir /b /ad {config_path()}\\v2ray_profiles\\subscriptions"
        #     self.v2ray_subscriptions = os.popen(cmd).read().strip().split("\n")
        # else:
        #     cmd = f"cd {config_path()}/v2ray_profiles/subscriptions;ls -d */"
        #     self.v2ray_subscriptions = (
        #         os.popen(cmd).read().strip().replace("/", "").split()
        #     )
        for subscription_name in self.v2ray_subscriptions:
            self.json_v2ray_subscription_configs = (
                self._referesh_v2ray_subscription_folder(subscription_name)
            )
        #######################################
        self.gost_subscriptions = []
        # if inside_windows():
        #     cmd = f"dir /b /ad {config_path()}\\gost_profiles\\subscriptions"
        #     self.gost_subscriptions = os.popen(cmd).read().strip().split("\n")
        # else:
        #     cmd = f"cd {config_path()}/gost_profiles/subscriptions;ls -d */"
        #     self.gost_subscriptions = (
        #         os.popen(cmd).read().strip().replace("/", "").split()
        #     )

        if inside_windows():
            path = f"{config_path()}\\gost_profiles\\subscriptions"
        else:
            path = f"{config_path()}/gost_profiles/subscriptions"
        self.check_folder_if_not_make(path)
        self.gost_subscriptions = [
            file for file in os.listdir(path) if os.path.isdir(os.path.join(path, file))
        ]

        print(self.gost_subscriptions)

        for subscription_name in self.gost_subscriptions:
            self.json_gost_subscription_configs = (
                self._referesh_gost_subscription_folder(subscription_name)
            )

    def extract_all_rows(self):
        rows = []
        ###############################
        for subscription_name in self.v2ray_subscriptions:
            i = 0
            for json_config in self.json_v2ray_subscription_configs:
                # try:

                row = self._extract_row_from_config_v2ray(json_config)
                # except Exception as e:
                #     continue
                row["remark"] = self.list_of_v2ray_subscription_configs[i]
                row["group"] = subscription_name
                rows.append(row)
                i += 1
        ####################################
        for subscription_name in self.gost_subscriptions:
            i = 0
            for json_config in self.json_gost_subscription_configs:
                row = self._extract_row_from_config_gost(json_config)
                row["remark"] = self.list_of_gost_subscription_configs[i]
                row["group"] = subscription_name
                rows.append(row)
                i += 1
        ####################################
        i = 0
        for json_config in self.json_v2ray_configs:
            row = self._extract_row_from_config_v2ray(json_config)
            row["remark"] = self.list_of_v2ray_configs[i]
            row["group"] = ""
            rows.append(row)
            i += 1
        ###########################################
        i = 0
        for json_config in self.json_gost_configs:
            row = self._extract_row_from_config_gost(json_config)
            row["remark"] = self.list_of_gost_configs[i]
            row["group"] = ""
            rows.append(row)
            i += 1
        #############################

        return rows

    def _extract_row_from_config_gost(self, data: json):
        row_data = {
            "protocol": data["remote_protocol"],
            "network": data["remote_protocol"],
            "server_address": data["remote_address"],
            "port": data["remote_port"],
            "security": data["remote_protocol"],
            "user_security": data["remote_protocol"],
        }
        return row_data

    def _extract_row_from_config_v2ray(self, data: json):
        protocol = data["outbounds"][0]["protocol"]
        network = data["outbounds"][0]["streamSettings"]["network"]
        user_security = ""
        if "trojan" in protocol:
            server_address = data["outbounds"][0]["settings"]["servers"][0]["address"]
            port = data["outbounds"][0]["settings"]["servers"][0]["port"]
        elif ("vless" in protocol) or ("vmess" in protocol):
            server_address = data["outbounds"][0]["settings"]["vnext"][0]["address"]
            port = data["outbounds"][0]["settings"]["vnext"][0]["port"]
            user_security = data["outbounds"][0]["settings"]["vnext"][0]["users"][0][
                "security"
            ]

        try:
            security = data["outbounds"][0]["streamSettings"]["security"]
        except:
            security = "none"

        row_data = {
            "protocol": protocol,
            "network": network,
            "server_address": server_address,
            "port": port,
            "security": security,
            "user_security": user_security,
        }
        return row_data

    def _referesh_v2ray_folder(self):
        # list_of_configs = (
        #     os.popen(
        #         f"dir {config_path()}\\v2ray_profiles /a-d /B"
        #         if inside_windows()
        #         else f"ls -p {config_path()}/v2ray_profiles | grep -v /"
        #     )
        #     .read()
        #     .split("\n")
        # )
        # list_of_configs = [x for x in list_of_configs if x]
        if inside_windows():
            path = f"{config_path()}\\v2ray_profiles"
        else:
            path = f"{config_path()}/v2ray_profiles"

        self.check_folder_if_not_make(path)

        list_of_configs = [
            file
            for file in os.listdir(path)
            if os.path.isfile(os.path.join(path, file))
        ]

        self.list_of_v2ray_configs = list_of_configs
        json_configs = []
        for config in list_of_configs:
            path = f"{config_path()}\\v2ray_profiles\\{config}"
            if not inside_windows:
                path = path.replace("\\", "/")

            json_configs.append(self._read_content_of_config_file(path))
        return json_configs

    def check_folder_if_not_make(self, path):
        if not os.path.exists(path):
            os.makedirs(path)
            print("Folder created successfully.")
        else:
            print("Folder already exists.")

    def _referesh_v2ray_subscription_folder(self, subscription_name):
        # list_of_configs = (
        #     os.popen(
        #         f"dir {config_path()}\\v2ray_profiles\\subscriptions\\{subscription_name} /a-d /B"
        #         if inside_windows()
        #         else f"ls -p  {config_path()}/v2ray_profiles/subscriptions/{subscription_name} | grep -v /"
        #     )
        #     .read()
        #     .split("\n")
        # )
        # list_of_configs = [x for x in list_of_configs if x]

        if inside_windows():
            path = (
                f"{config_path()}\\v2ray_profiles\\subscriptions\\{subscription_name}"
            )
        else:
            path = f"{config_path()}/v2ray_profiles/subscriptions/{subscription_name}"

        list_of_configs = [
            file
            for file in os.listdir(path)
            if os.path.isfile(os.path.join(path, file))
        ]

        self.list_of_v2ray_subscription_configs = list_of_configs
        json_configs = []
        for config in list_of_configs:
            path = f"{config_path()}\\v2ray_profiles\\subscriptions\\{subscription_name}\\{config}"
            if not inside_windows:
                path = path.replace("\\", "/")
            temp = self._read_content_of_config_file(path)
            json_configs.append(temp)
        return json_configs

    def _referesh_gost_subscription_folder(self, subscription_name):
        # list_of_configs = (
        #     os.popen(
        #         f"dir {config_path()}\\gost_profiles\\subscriptions\\{subscription_name} /a-d /B"
        #         if inside_windows()
        #         else f"ls -p  {config_path()}/gost_profiles/subscriptions/{subscription_name} | grep -v /"
        #     )
        #     .read()
        #     .split("\n")
        # )
        # list_of_configs = [x for x in list_of_configs if x]
        if inside_windows():
            path = f"{config_path()}\\gost_profiles\\subscriptions\\{subscription_name}"
        else:
            path = f"{config_path()}/gost_profiles/subscriptions/{subscription_name}"

        list_of_configs = [
            file
            for file in os.listdir(path)
            if os.path.isfile(os.path.join(path, file))
        ]
        self.list_of_gost_subscription_configs = list_of_configs
        json_configs = []
        for config in list_of_configs:
            path = f"{config_path()}\\gost_profiles\\subscriptions\\{subscription_name}\\{config}"
            if not inside_windows:
                path = path.replace("\\", "/")
            temp = self._read_content_of_config_file(path)
            json_configs.append(temp)
        return json_configs

    def _referesh_gost_folder(self):
        # list_of_configs = (
        #     os.popen(f"dir {config_path()}\\gost_profiles /a-d /B").read().split("\n")
        #     if inside_windows()
        #     else os.popen(f"ls -p {config_path()}/gost_profiles | grep -v /")
        #     .read()
        #     .split("\n")
        # )
        # list_of_configs = [x for x in list_of_configs if x]

        if inside_windows():
            path = f"{config_path()}\\gost_profiles"
        else:
            path = f"{config_path()}/gost_profiles"

        self.check_folder_if_not_make(path)
        list_of_configs = [
            file
            for file in os.listdir(path)
            if os.path.isfile(os.path.join(path, file))
        ]

        self.list_of_gost_configs = list_of_configs
        json_configs = []
        for config in list_of_configs:
            # print("gost config:",config)
            path = f"{config_path()}\\gost_profiles\\{config}"
            if not inside_windows:
                path = path.replace("\\", "/")
            json_configs.append(self._read_content_of_config_file(path))
        return json_configs

    def _read_content_of_config_file(self, file_path) -> json:
        # print(f"221 this is text: {file_path}*")
        if not inside_windows():
            file_path = file_path.replace("\\", "/")
        with open(f"{file_path}", "r", encoding="utf-8") as json_file:
            # Load the JSON data
            text = json_file.read()
        data = json.loads(text, strict=False)
        return data


if __name__ == "__main__":
    RefereshTableContent()
    while True:
        time.sleep(10)



import sys

import PySimpleGUI as psg

sys.path.append("v2rayp")
from libs.in_win import FactorySetting

psg.set_options(font=("Arial Bold", 11))


page_data = dict()
page_data["keep_top"] = True
page_data["close_to_tray"] = False
page_data["auto_connect"] = False
page_data["start_minimized"] = False
page_data["cloudflare_address"] = "bruce.ns.cloudflare.com"
page_data["cloudflare_port"] = "2087"
page_data["segmentation_timeout"] = 5
page_data["num_of_fragments"] = 100
page_data["beep"] = True


class SettingGUI:
    def __init__(self, page_data=page_data) -> None:
        self.settings = page_data
        self.layout = [
            [self.generate_top_parts()],
            [psg.HorizontalSeparator()],
            [self.factory_reset_layout()],
            [psg.HorizontalSeparator()],
            [self.generate_middle_parts()],
            [psg.HorizontalSeparator()],
            [self.generate_buttons()],
        ]

    def getLayout(self) -> list:
        return self.layout

    def factory_reset_layout(self):
        row = [
            [psg.Text("Factory Reset:", font=("Arial Bold", 14))],
            [
                psg.Checkbox(
                    "Factory:\t\t",
                    default=bool(self.settings["keep_top"]),
                    key="keep_top",
                ),
            ],
        ]
        return row

    def generate_top_parts(self):
        layout = [
            [psg.Text("Settings", justification="center", font=("Arial Bold", 18))],
            [psg.HorizontalSeparator()],
            [
                psg.Text("Beep:\t"),
                psg.Checkbox(
                    text="",
                    default=self.settings["beep"] if "beep" in self.settings else False,
                    key="beep",
                ),
            ],
            [psg.HorizontalSeparator()],
            [psg.Text("Configs:", font=("Arial Bold", 14))],
            [psg.Button("Factory Reset", key="factory")],
            [
                psg.Checkbox(
                    "Minimize to tray on close:\t\t",
                    default=bool(self.settings["close_to_tray"]),
                    key="close_to_tray",
                ),
            ],
            [
                psg.Checkbox(
                    "Auto connect on start:\t\t",
                    default=bool(self.settings["auto_connect"]),
                    key="auto_connect",
                ),
            ],
            [
                psg.Checkbox(
                    "Start minimized:\t\t",
                    default=bool(self.settings["start_minimized"]),
                    key="start_minimized",
                ),
            ],
        ]
        return layout

    def generate_middle_parts(self):
        layout = [
            [psg.Text("Segmentation Settings:", font=("Arial Bold", 14))],
            [
                psg.Text("Cloudflare Address:\t"),
                psg.InputText(
                    default_text=self.settings["cloudflare_address"],
                    key="cloudflare_address",
                    size=(20, 10),
                ),
            ],
            # [
            #     psg.Text("Cloudflare Port:\t\t"),
            #     psg.InputText(
            #         size=(20, 10),
            #         default_text=self.settings["cloudflare_port"],
            #         key="cloudflare_port",
            #     ),
            # ],
            [
                psg.Text("Segmentation Timeout:\t"),
                psg.InputText(
                    size=(20, 10),
                    default_text=self.settings["segmentation_timeout"],
                    key="segmentation_timeout",
                ),
            ],
            [
                psg.Text("Number of Segmentation:\t"),
                psg.InputText(
                    size=(20, 10),
                    default_text=self.settings["num_of_fragments"],
                    key="num_of_fragments",
                ),
            ],
        ]
        return layout

    def generate_buttons(self):
        layout = [
            psg.Button(key="confirm", button_text="Confirm"),
            psg.Button(key="cancel", button_text="Cancel"),
        ]
        return layout

    def save_gui(self):
        self.settings["keep_top"] = self.window["keep_top"].get()
        self.settings["close_to_tray"] = self.window["close_to_tray"].get()
        self.settings["auto_connect"] = self.window["auto_connect"].get()
        self.settings["start_minimized"] = self.window["start_minimized"].get()
        self.settings["cloudflare_address"] = self.window["cloudflare_address"].get()
        # self.settings["cloudflare_port"] = self.window["cloudflare_port"].get()
        self.settings["num_of_fragments"] = self.window["num_of_fragments"].get()
        self.settings["segmentation_timeout"] = self.window[
            "segmentation_timeout"
        ].get()
        self.settings["beep"] = self.window["beep"].get()

    def close(self):
        self.window.close()

    def start_window(self) -> dict:
        layout = self.getLayout()
        self.window = psg.Window(
            "Vmess",
            layout,
            # size=(600, 790),
            resizable=False,
            # no_titlebar=True,
            keep_on_top=True,
            grab_anywhere=True,
        )
        while True:
            event, _ = self.window.read()
            # print("event:", event, "values:", values)
            if event == psg.WIN_CLOSED:
                break
            if "confirm" in event:
                self.save_gui()
                break
            elif "cancel" in event:
                self.settings = None
                break
            elif "factory" in event:
                response = psg.popup_yes_no(
                    "***Warning***", "Are you sure?", keep_on_top=True
                )
                print(response)
                if response == "Yes":
                    self.factory_reset()
                    # os.execv(
                    #     sys.executable, [os.path.basename(sys.executable)] + sys.argv
                    # )
                    self.window.close()
                    return "factory_reset"
                break

        self.window.close()
        return self.settings

    def factory_reset(self):
        FactorySetting.delete_config_folder()


if __name__ == "__main__":
    print(SettingGUI(page_data).start_window())



import PySimpleGUI as psg
import sys

sys.path.append("v2rayp")
psg.set_options(font=("Arial Bold", 11))
page_data = dict()
page_data["remark"] = ""
page_data["server_address"] = ""
page_data["port"] = ""
page_data["uuid"] = ""
page_data["user_security"] = ""
page_data["grpc_serviceName"] = ""
page_data["network"] = ""
page_data["security"] = ""
page_data["sni"] = ""
page_data["fingerprint"] = ""
page_data["publicKey"] = ""
page_data["shortId"] = ""
page_data["spiderX"] = ""


class VmessGUI:
    def __init__(self, page_data: dict = page_data) -> None:
        self.page_data = page_data
        self.layout = [
            [self.generate_top_parts()],
            [psg.HorizontalSeparator()],
            [self.generate_middle_parts()],
            [psg.HorizontalSeparator()],
            [self.generate_bottom_parts()],
            [psg.HorizontalSeparator()],
            [self.generate_ending_parts()],
            [psg.HorizontalSeparator()],
            [self.generate_buttons()],
        ]

    def getLayout(self) -> list:
        return self.layout

    def save_return(self):
        self.page_data["remark"] = self.page_data["pas"] = self.window["alias"].get()
        self.page_data["server_address"] = self.window["address"].get()
        self.page_data["port"] = self.window["port"].get()
        self.page_data["uuid"] = self.window["uuid"].get()
        self.page_data["user_security"] = self.window["encryptionmethod"].get()
        # self.page_data["grpc_serviceName"] = "imi"
        self.page_data["network"] = self.window["transport_protocol"].get()
        self.page_data["security"] = self.window["tls"].get()
        self.page_data["sni"] = self.window["sni"].get()
        self.page_data["fingerprint"] = self.window["fingerprint"].get()
        self.page_data["publicKey"] = self.window["publickey"].get()
        self.page_data["shortId"] = self.window["shortid"].get()
        self.page_data["spiderX"] = self.window["spiderx"].get()
        return self.page_data

    def generate_top_parts(self):
        layout = [
            [psg.Text("Vmess", justification="center", font=("Arial Bold", 18))],
            [psg.HorizontalSeparator()],
            [psg.Text("Servers")],
            [
                psg.Text("Alias:\t\t"),
                psg.InputText(default_text=self.page_data["remark"], key="alias"),
            ],
            [
                psg.Text("Address:\t\t"),
                psg.InputText(
                    default_text=self.page_data["server_address"], key="address"
                ),
            ],
            [
                psg.Text("Port:\t\t"),
                psg.InputText(default_text=self.page_data["port"], key="port"),
            ],
        ]
        return layout

    def generate_middle_parts(self):
        combo_values = ["auto", "aes-128-gcm", "chacha20-poly1305"]
        layout = [
            [
                psg.Text("UUID:\t\t"),
                psg.InputText(default_text=self.page_data["uuid"], key="uuid"),
            ],
            [psg.Text("AlterID:\t\t"), psg.InputText(key="alterid")],
            [
                psg.Text("Encryption\nMethod:\t\t"),
                psg.Combo(
                    default_value=self.page_data["user_security"],
                    values=combo_values,
                    key="encryptionmethod",
                ),
            ],
        ]
        return layout

    def generate_bottom_parts(self):
        protocols = ["ws", "tcp", "kcp", "grpc", "h2", "quic"]
        layout = [
            [psg.Text("Transport", font=("Arial Bold", 14))],
            [
                psg.Text("Transport\nProtocol:\t"),
                psg.Combo(
                    default_value=self.page_data["network"],
                    values=protocols,
                    key="transport_protocol",
                ),
            ],
            [
                psg.Text(
                    font=("", 11), text="Camouflage\nType or GRPC\nServiceName:\t"
                ),
                psg.InputText(
                    default_text=self.page_data["grpc_serviceName"]
                    if self.page_data["network"] == "grpc"
                    else "",
                    key="camouflage_type",
                ),
            ],
            [psg.Text("Camouflage\ndomain (host):\t"), psg.InputText(key="host")],
            [psg.Text("Path:\t\t"), psg.InputText(key="path")],
        ]
        return layout

    def generate_ending_parts(self):
        # tls = ["reality", "tls"]
        layout = [
            [
                psg.Text("Tls:\t\t"),
                psg.Combo(
                    default_value=self.page_data["security"],
                    values=["none", "tls", "reality"],
                    key="tls",
                ),
            ],
            [
                psg.Text("SNI:\t\t"),
                psg.InputText(default_text=self.page_data["uuid"], key="sni"),
            ],
            [
                psg.Text("Fingerprint:\t"),
                psg.Combo(
                    default_value=self.page_data["fingerprint"]
                    if "fingerprint" in self.page_data
                    else "",
                    values=["firefox", "chrome", "android"],
                    key="fingerprint",
                ),
            ],
            [
                psg.Text("Public\nKey:\t\t"),
                psg.InputText(
                    default_text=self.page_data["publicKey"]
                    if "publicKey" in self.page_data
                    else "",
                    key="publickey",
                ),
            ],
            [
                psg.Text("ShortID:\t\t"),
                psg.InputText(
                    default_text=self.page_data["shortId"]
                    if "shortId" in self.page_data
                    else "",
                    key="shortid",
                ),
            ],
            [
                psg.Text("SpiderX:\t\t"),
                psg.InputText(
                    default_text=self.page_data["spiderX"]
                    if "spiderX" in self.page_data
                    else "",
                    key="spiderx",
                ),
            ],
        ]
        return layout

    def generate_buttons(self):
        layout = [
            psg.Button(key="confirm", button_text="Confirm"),
            psg.Button(key="cancel", button_text="Cancel"),
        ]
        return layout

    def start_window(self):
        layout = self.getLayout()
        self.window = psg.Window(
            "Vmess",
            layout,
            # size=(600, 790),
            resizable=True,
            no_titlebar=True,
            grab_anywhere=True,
            keep_on_top=True,
        )
        while True:
            event, values = self.window.read()
            # print("event:", event, "values:", values)

            if "confirm" in event:
                page_data = self.save_return()
                break
            elif "cancel" in event or event == psg.WIN_CLOSED:
                page_data = None
                break
        self.window.close()
        return page_data


if __name__ == "__main__":
    VmessGUI().start_window()



import functools
import os
import time
from threading import Thread

import requests
from libs.in_win import config_path


class NetTools:
    @staticmethod
    def timeout(timeout):
        def deco(func):
            @functools.wraps(func)
            def wrapper(*args, **kwargs):
                res = [
                    Exception(
                        "1366_function [%s] timeout [%s seconds] exceeded!"
                        % (func.__name__, timeout)
                    )
                ]

                def newTargetFunc():
                    try:
                        res[0] = func(*args, **kwargs)
                    except Exception as e:
                        res[0] = e

                t = Thread(target=newTargetFunc)
                t.daemon = True
                try:
                    t.start()
                    t.join(timeout)
                except Exception as je:
                    print("error starting thread")
                    raise je
                ret = res[0]
                if isinstance(ret, BaseException):
                    raise ret
                return ret

            return wrapper

        return deco

    @staticmethod
    def measure_ping_through_socks_proxy(
        proxy_address, proxy_address_port, target_host
    ):
        """Returns delay in milli-seconds"""
        # Set up the SOCKS proxy

        start_time = time.time()
        cmd = f"curl -s  --socks5-hostname {proxy_address}:{proxy_address_port} {target_host}"
        # print(cmd)
        out = os.popen(cmd).read()

        end_time = time.time()

        # Calculate the ping delay
        ping_delay = int(1000 * (end_time - start_time))
        return ping_delay

    @staticmethod
    # @timeout(5)
    def is_connected_to_internet(address="https://www.yahoo.com", lport=7595):
        proxy = {
            "http": f"socks5://127.0.0.1:{lport}",
            "https": f"socks5://127.0.0.1:{lport}",
        }

        try:
            # Attempt to send a request to a reliable website (e.g., Google) using the proxy
            response = requests.get(address, proxies=proxy, timeout=5)

            # Check if the response status code is 200 (successful)
            if response.status_code == 200:
                return True
            else:
                return False
        except requests.RequestException:
            # If an exception is raised, it means the request failed (no internet connection)
            return False


if __name__ == "__main__":
    # Replace these values with your SOCKS proxy address and the target host and port you want to ping
    # proxy_address = "127.0.0.1"
    # proxy_address_port = 7595
    # target_host = "https://google.com"
    # target_port = 80

    # ping_delay = NetTools.measure_ping_through_socks_proxy(
    #     proxy_address, proxy_address_port, target_host
    # )
    # if ping_delay is not None:
    #     print(
    #         f"Ping delay to {target_host}:{target_port} via the proxy: {ping_delay:.3f} milli-seconds"
    #     )
    print(NetTools.is_connected_to_internet())



import ctypes
import functools
import multiprocessing
import os
import random
import socket
import sys
import threading
import time
from threading import Thread

# multiprocessing.freeze_support()


class GFW_Interface:
    local_socket = ""

    def __init__(
        self,
        num_fragment,
        listen_PORT,
        Cloudflare_IP,
        Cloudflare_port,
        from_cmd=False,
        GFW_Timeout=10,
    ):
        self.num_fragment = num_fragment
        self.listen_PORT = listen_PORT  # pyprox listening to 127.0.0.1:listen_PORT
        self.Cloudflare_IP = Cloudflare_IP
        self.Cloudflare_port = Cloudflare_port
        self.timeout_in_socksets = GFW_Timeout
        # self.mainThread = threading.Thread(target=self.start_tunnel)
        # self.mainThread.start()
        self.init_localsocket()
        self.mainThread = threading.Thread(target=self.start_tunnel)
        self.mainThread.daemon = True
        self.mainThread.start()
        # self.mainThread.join()
        # if from_cmd:
        #     self.mainThread = threading.Thread(target=self.start_tunnel)
        #     self.mainThread.daemon = True
        #     self.mainThread.start()
        #     self.mainThread.join()
        # else:
        #     self.mainThread = multiprocessing.Process(target=self.start_tunnel)
        #     self.mainThread.start()

    def init_localsocket(self):
        self.local_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        # self.local_socket.settimeout(self.timeout_in_socksets)
        self.local_socket.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
        self.local_socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)

    def stop(self):
        print("**Stop GFW is called***")
        self.loop = False
        # self.local_socket.setblocking(False)
        try:
            self.local_socket.close()
        except:
            print("error closing..")
        # GFW_thread_id = self.mainThread.ident
        # ctypes.pythonapi.PyThreadState_SetAsyncExc(
        #     GFW_thread_id, ctypes.py_object(SystemExit)
        # )
        # self.mainThread.join(1)

        try:
            self.local_socket.close()
        except:
            pass

        try:
            self.mainThread.kill()
            self.mainThread.terminate()

        except:
            pass
        # self.mainThread.join()
        print(f"Subprocess {self.mainThread.name} alive: {self.mainThread.is_alive}")
        # self.mainThread.join(2)

    @staticmethod
    def timeout(timeout):
        def deco(func):
            @functools.wraps(func)
            def wrapper(*args, **kwargs):
                res = [
                    Exception(
                        "1366_function [%s] timeout [%s seconds] exceeded!"
                        % (func.__name__, timeout)
                    )
                ]

                def newTargetFunc():
                    try:
                        res[0] = func(*args, **kwargs)
                    except Exception as e:
                        res[0] = e

                t = Thread(target=newTargetFunc)
                t.daemon = True
                try:
                    t.start()
                    t.join(timeout)
                except Exception as je:
                    print("error starting thread")
                    raise je
                ret = res[0]
                if isinstance(ret, BaseException):
                    raise ret
                return ret

            return wrapper

        return deco

    def start_tunnel(self):
        self.loop = True
        # Define the local listening address and port
        local_host = "0.0.0.0"
        local_port = self.listen_PORT

        # # Define the remote server address and port
        remote_host = self.Cloudflare_IP
        remote_port = self.Cloudflare_port
        print(
            f"Main func is started-> local::{local_host}:{local_port}\nremote::{remote_host}:{remote_port} "
        )
        while self.loop:
            try:
                # Create the local listening socket

                # try:
                self.init_localsocket()
                self.local_socket.bind((local_host, local_port))
                self.local_socket.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
                self.local_socket.listen(20)

            except Exception as problem:
                print(f"High level exeception: {problem}")
            #

            print(f"Listening on {local_host}:{local_port}...")
            while self.loop:
                print(f"number of threads: {len(threading.enumerate())}")
                print("waiting for new connection...")
                try:
                    time.sleep(0.1)
                    client_socket, client_address = self.local_socket.accept()
                    client_socket.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
                    client_socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
                    print(
                        f"Accepted connection from {client_address[0]}:{client_address[1]}"
                    )
                    remote_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                    remote_socket.connect((remote_host, remote_port))
                    remote_socket.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
                    remote_socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
                    print(f"Connected to remote server {remote_host}:{remote_port}")
                    #################
                    self.trans_rec(client_socket, remote_socket)
                    ##################
                except Exception as problem:
                    print(f"Error 770: {problem}")
                    if "completed immediately" in str(problem):
                        print("continue")
                        continue

                    # if "1366" in str(e):
                    #     remote_socket.close()
                    #     continue
                    try:
                        client_socket.close()
                        remote_socket.close()
                    except Exception as e:
                        print(f"Error 990: {e}")
                        continue
                    print("\n***Connection closed***\n")
                    # time.sleep(0.1)

    # @timeout(1)
    def trans_rec(self, client_socket, remote_socket):
        client_to_server_thread = self.forward_data(
            client_socket, remote_socket, "client_to_server"
        )
        random_number = random.random() * 0.5
        time.sleep(random_number)
        server_to_client_thread = self.forward_data(
            remote_socket, client_socket, "server_to_client"
        )

        # client_to_server_thread.join(0.5)
        # server_to_client_thread.join()

    def forward_data(
        self, src_socket: socket.socket, dst_socket: socket.socket, mode: str
    ):
        def forward():
            loop_counter = 1
            socketsize = self.generate_random_number_about(1000, 0.75)
            rand = self.generate_random_number_about(self.timeout_in_socksets, 0.2)

            @self.timeout(rand)
            def rec():
                return src_socket.recv(socketsize)

            while self.loop:
                try:
                    data = rec()
                    # print(f"while counter : {while_counter}")
                    # print(f"len data is: {len(data)}")
                    if data:
                        if loop_counter <= 1 and mode == "client_to_server":
                            self.send_data_in_fragment(data, dst_socket)
                        else:
                            # dst_socket.sendall(data)
                            dst_socket.sendall(data)
                    else:
                        raise Exception("No data execption***")
                except Exception as e:
                    print(f"Err occured: in {mode} {e}")
                    try:
                        src_socket.close()
                        dst_socket.close()
                    except:
                        pass
                    break
                loop_counter += 1

        thread = threading.Thread(target=forward)
        thread.daemon = True
        thread.start()
        return thread

    def generate_random_number_about(self, number, ratio=0.15):
        deviation = number * ratio
        min_value = int(number - deviation)
        max_value = int(number + deviation)
        return random.randint(min_value, max_value)

    # @timeout(2)
    def send_data_in_fragment(self, data: list, sock: socket.socket):
        L_data = len(data)
        rand = self.generate_random_number_about(self.num_fragment)
        indices = random.sample(
            range(1, L_data - 1),
            rand - 1,
        )
        indices.sort()
        print("indices=", indices)

        i_pre = 0
        for i in indices:
            fragment_data = data[i_pre:i]
            i_pre = i
            # sock.sendall(fragment_data)
            sock.sendall(fragment_data)
            # time.sleep(0.1)

        fragment_data = data[i_pre:L_data]
        sock.sendall(fragment_data)
        print("----------finish------------")


# Start the tunne
if __name__ == "__main__":
    multiprocessing.freeze_support()
    try:
        num_fragment = int(sys.argv[4])
        listen_PORT = int(sys.argv[2])  # pyprox listening to 127.0.0.1:listen_PORT
        Cloudflare_IP = sys.argv[1]
        Cloudflare_port = int(sys.argv[3])
        GFW_Interface(
            num_fragment, listen_PORT, Cloudflare_IP, Cloudflare_port, True, 5
        )
    except:
        listen_PORT = 2500
        num_fragment = 75
        Cloudflare_IP = "bruce.ns.cloudflare.com"
        Cloudflare_port = 8443
        GFW_Interface(
            num_fragment, listen_PORT, Cloudflare_IP, Cloudflare_port, True, 10
        )
    while True:
        time.sleep(10)
        ##########################################################
        #########################################################



import os

from setuptools import find_packages, setup

from v2rayp.__version__ import __version__


def package_files(directory):
    paths = []
    for path, directories, filenames in os.walk(directory):
        for filename in filenames:
            if ".pyc" in filename:
                continue
            paths.append(os.path.join("..", path, filename))
    return paths


extra_files = package_files("v2rayp")


setup(
    name="v2rayp",  # How you named your package folder (MyLib)
    packages=find_packages(exclude=[".pyc"]),  # Chose the same as "name"
    package_data={"": extra_files},
    version=__version__,  # Start with a small number and increase it with every change you make
    license="MIT",  # Chose a license from here: https://help.github.com/articles/licensing-a-repository
    description="An efficient and versatile multiplatform project, serving as a v2rayn substitute for Windows, while seamlessly functioning on Mac and Linux, featuring fragmentation support and multi-threaded capabilities.",  # Give a short description about your library
    author="iBlockchain",  # Type in your name
    author_email="iman.minstry@gmail.com",  # Type in your E-Mail
    url="https://github.com/iblockchaincyberchain/v2rayp",  # Provide either the link to your github or to your website
    # download_url="https://github.com/iblockchaincyberchain/V2rayP/releases/download/untagged-56f596c21486f79c4a17/V2rayP.zip",  # I explain this later on
    keywords=[
        "Fragmentation",
        "V2Ray",
        "multiplatform",
        "VPN",
        "Proxy",
    ],  # Keywords that define your package best
    install_requires=[  # I get to this in a second
        "PySimpleGUI",
        "psgtray",
        "requests",
        "pyperclip",
        "psutil",
        "qrcode",
        "setuptools",
    ],
    classifiers=[
        "Development Status :: 3 - Alpha",  # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
        "Intended Audience :: Developers",  # Define that your audience are developers
        "Topic :: Software Development :: Build Tools",
        "License :: OSI Approved :: MIT License",  # Again, pick a license
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
    ],
)



import os
import sys

sys.path.append("v2rayp")
import PySimpleGUI as psg

psg.set_options(font=("Arial Bold", 11))
page_data = dict()
page_data["remark"] = ""
page_data["password"] = ""
page_data["serverName"] = ""
page_data["server_address"] = ""
page_data["port"] = ""
page_data["uuid"] = ""
page_data["user_security"] = ""
page_data["grpc_serviceName"] = ""
page_data["network"] = ""
page_data["security"] = ""
page_data["sni"] = ""
page_data["fingerprint"] = ""
page_data["publicKey"] = ""
page_data["shortId"] = ""
page_data["spiderX"] = ""


class TrojanGUI:
    def __init__(self, page_data: dict = page_data) -> None:
        self.page_data = page_data
        # print(page_data)
        self.layout = [
            [self.generate_top_parts()],
            [psg.HorizontalSeparator()],
            [self.generate_middle_parts()],
            [psg.HorizontalSeparator()],
            [self.generate_bottom_parts()],
            [psg.HorizontalSeparator()],
            [self.generate_ending_parts()],
            [psg.HorizontalSeparator()],
            [self.generate_buttons()],
        ]

    def getLayout(self) -> list:
        return self.layout

    def generate_top_parts(self):
        layout = [
            [psg.Text("Trojan", justification="center", font=("Arial Bold", 18))],
            [psg.HorizontalSeparator()],
            [psg.Text("Servers")],
            [
                psg.Text("Alias:\t\t"),
                psg.InputText(default_text=self.page_data["remark"], key="alias"),
            ],
            [
                psg.Text("Address:\t\t"),
                psg.InputText(
                    default_text=self.page_data["server_address"], key="address"
                ),
            ],
            [
                psg.Text("Port:\t\t"),
                psg.InputText(default_text=self.page_data["port"], key="port"),
            ],
        ]
        return layout

    def generate_middle_parts(self):
        encryption_methods = ["auto", "aes-128-gcm", "chacha20-poly1305"]
        layout = [
            [
                psg.Text("Password:\t"),
                psg.InputText(
                    default_text=self.page_data["password"],
                    key="password",
                ),
            ],
            [
                psg.Text("Flow:\t\t"),
                psg.Combo(
                    values=["none", "xtls-rprx-vision", "xtls-rprx-vision-udp443"],
                    key="flow",
                ),
            ],
            [
                psg.Text("Encryption\nMethod:\t\t"),
                psg.Combo(
                    default_value=self.page_data["user_security"],
                    values=encryption_methods,
                    key="encryptionmethod",
                ),
            ],
        ]
        return layout

    def generate_bottom_parts(self):
        protocols = ["ws", "tcp", "kcp", "grpc", "h2", "quic"]
        layout = [
            [psg.Text("Transport")],
            [
                psg.Text("Transport\nProtocol:\t"),
                psg.Combo(
                    default_value=self.page_data["network"],
                    values=protocols,
                    key="transport_protocol",
                ),
            ],
            [psg.Text("Camouflage Type:\t"), psg.InputText(key="camouflage_type")],
            [psg.Text("Camouflage\ndomain (host):\t"), psg.InputText(key="host")],
            [psg.Text("Path:\t\t"), psg.InputText(key="path")],
            # [
            #     psg.Text("Tls:\t\t"),
            #     psg.Combo(
            #         default_value=self.page_data["security"],
            #         values=["none", "tls", "reality"],
            #         key="tls",
            #     ),
            # ],
        ]
        return layout

    def generate_ending_parts(self):
        tls = ["reality", "tls"]
        layout = [
            [
                psg.Text("TLS:\t\t"),
                psg.Combo(
                    default_value=self.page_data["security"]
                    if "security" in self.page_data
                    else self.page_data["tls"],
                    values=tls,
                    key="tls",
                ),
            ],
            [
                psg.Text("SNI:\t\t"),
                psg.InputText(
                    default_text=self.page_data["serverName"], key="serverName"
                ),
            ],
            [
                psg.Text("Fingerprint:\t"),
                psg.Combo(
                    default_value=self.page_data["fingerprint"],
                    values=["firefox", "chrome", "android"],
                    key="fingerprint",
                ),
            ],
        ]

        if self.page_data["security"] == "reality":
            lay_bott = [
                [
                    psg.Text("Public Key:\t"),
                    psg.InputText(
                        default_text=self.page_data["publicKey"], key="publickey"
                    ),
                ],
                [
                    psg.Text("ShortID:\t\t"),
                    psg.InputText(
                        default_text=self.page_data["shortId"], key="shortid"
                    ),
                ],
                [
                    psg.Text("SpiderX:\t\t"),
                    psg.InputText(
                        default_text=self.page_data["spiderX"], key="spiderx"
                    ),
                ],
            ]
            layout.append(lay_bott)

        return layout

    def generate_buttons(self):
        layout = [
            psg.Button(key="confirm", button_text="Confirm"),
            psg.Button(key="cancel", button_text="Cancel"),
        ]
        return layout

    def start_window(self):
        layout = self.getLayout()
        self.window = psg.Window(
            "Trojan",
            layout,
            # size=(600, 750),
            resizable=True,
            no_titlebar=True,
            grab_anywhere=True,
            keep_on_top=True,
        )
        while True:
            event, values = self.window.read()
            # print("event:", event, "values:", values)
            if event == psg.WIN_CLOSED or "cancel" in event:
                break
            elif "confirm" in event:
                self.page_data["remark"] = self.page_data["ps"] = self.window[
                    "alias"
                ].get()
                self.page_data["server_address"] = self.window["address"].get()
                self.page_data["port"] = self.window["port"].get()
                self.page_data["password"] = self.window["password"].get()
                self.page_data["flow"] = self.window["flow"].get()
                self.page_data["user_security"] = self.window["encryptionmethod"].get()
                self.page_data["network"] = self.window["transport_protocol"].get()
                self.page_data["camouflage_type"] = self.window["camouflage_type"].get()
                self.page_data["host"] = self.window["host"].get()
                self.page_data["path"] = self.window["path"].get()
                self.page_data["security"] = self.page_data["tls"] = self.window[
                    "tls"
                ].get()
                self.page_data["serverName"] = self.window["serverName"].get()
                self.page_data["serverName"] = self.window["serverName"].get()
                self.page_data["fingerprint"] = self.window["fingerprint"].get()
                self.page_data["publicKey"] = self.window["publickey"].get()
                self.page_data["shortId"] = self.window["shortid"].get()
                self.page_data["spiderX"] = self.window["spiderx"].get()

                break

        self.window.close()
        print(self.page_data)
        return self.page_data


if __name__ == "__main__":
    TrojanGUI().start_window()



import json
import sys

sys.path.append("v2rayp")

import PySimpleGUI as psg
from libs.in_win import config_path, inside_windows

psg.set_options(font=("Arial Bold", 11))
page_data = dict()
page_data["remote_protocol"] = ""
page_data["remote_address"] = ""
page_data["remote_port"] = ""
page_data["remote_user"] = ""
page_data["remote_password"] = ""
page_data["remark"] = ""


class GostGUI:
    def __init__(self, pname, subscription="") -> None:
        if pname:
            self.pname = pname
            self.subscription = subscription
            self.load_json_data()

        else:
            self.page_data = page_data
            self.pname = f'{page_data["remote_protocol"]}_{page_data["remote_port"]}'
        self.layout = [
            [self.generate_top_parts()],
            [psg.HorizontalSeparator()],
            # [self.generate_middle_parts()],
            # [psg.HorizontalSeparator()],
            # [self.generate_bottom_parts()],
            # [psg.HorizontalSeparator()],
            # [self.generate_ending_parts()],
            # [psg.HorizontalSeparator()],
            [self.generate_buttons()],
        ]

    def getLayout(self) -> list:
        return self.layout

    def generate_top_parts(self):
        layout = [
            [psg.Text("Gost", justification="center", font=("Arial Bold", 18))],
            [psg.HorizontalSeparator()],
            [psg.Text(f'{self.page_data["remark"]}')],
            [
                psg.Text("Protocol:\t\t"),
                psg.InputText(
                    default_text=self.page_data["remote_protocol"], key="protocol"
                ),
            ],
            [
                psg.Text("Address:\t\t"),
                psg.InputText(
                    default_text=self.page_data["remote_address"], key="address"
                ),
            ],
            [
                psg.Text("Port:\t\t"),
                psg.InputText(default_text=self.page_data["remote_port"], key="port"),
            ],
            [
                psg.Text("Username:\t"),
                psg.InputText(
                    default_text=self.page_data["remote_user"], key="username"
                ),
            ],
            [
                psg.Text("Password:\t"),
                psg.InputText(
                    default_text=self.page_data["remote_password"], key="password"
                ),
            ],
        ]
        return layout

    def generate_buttons(self):
        layout = [
            psg.Button(key="confirm", button_text="Confirm"),
            psg.Button(key="cancel", button_text="Cancel"),
        ]
        return layout

    def start_window(self):
        layout = self.getLayout()
        self.window = psg.Window(
            "Gost",
            layout,
            # size=(600, 790),
            resizable=True,
            no_titlebar=True,
            grab_anywhere=True,
            keep_on_top=True,
        )
        while True:
            event, values = self.window.read()
            # print("event:", event, "values:", values)
            if event == psg.WIN_CLOSED or "cancel" in event:
                self.page_data = None
                break
            elif "confirm" in event:
                self.gether_data()

                break
        self.window.close()
        return self.page_data

    def gether_data(self):
        self.page_data["remote_protocol"] = self.window["protocol"].get()
        self.page_data["remote_address"] = self.window["address"].get()
        self.page_data["remote_port"] = self.window["port"].get()
        self.page_data["remote_user"] = self.window["username"].get()
        self.page_data["remote_password"] = self.window["password"].get()

    def load_json_data(self):
        subscription_path = ""

        if len(self.subscription) > 1:
            subscription_path = f"\\subscriptions\\{self.subscription}"
        with open(
            f"{config_path()}\\gost_profiles{subscription_path}\\{self.pname}",
            "r",
        ) as json_file:
            # Write the JSON data to the file
            self.page_data = json.load(json_file)
        self.page_data["remark"] = self.pname


if __name__ == "__main__":
    GostGUI(None).start_window()
